# Comodo AntiVirus 2.0
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def ExecuteStr(self, cmd, args):
    log.ThugLogging.add_behavior_warn('[Comodo AntiVirus ActiveX] Trying to execute: ' + cmd + ' ' + args)
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Comodo AntiVirus ActiveX",
                                      "Trying to execute",
                                      forward = False,
                                      data = {
                                                "command": cmd,
                                                "args"   : args
                                             }
                                      )



#!/usr/bin/env python

from .Node import Node
from .Text import Text


class CDATASection(Text):
    def __repr__(self):
        return "<CDATA '%s' at 0x%08X>" % (self.tag, id(self))

    @property
    def nodeName(self):
        return "#cdata-section"

    @property
    def nodeType(self):
        return Node.CDATA_SECTION_NODE



# BaiduBar.dll ActiveX DloadDS() Remote Code Execution Vulnerability
# BUGTRAQ  ID: 25121

import logging

log = logging.getLogger("Thug")


def DloadDS(self, arg0, arg1, arg2):
    if str(arg0).lower().find(".cab") != -1:
        log.ThugLogging.add_behavior_warn('[BaiduBar.dll ActiveX] DloadDS function trying to download %s' % (arg0, ))
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "BaiduBar.dll ActiveX",
                                          "DloadDS function trying to download",
                                          data = {
                                                    "url": arg0
                                                 },
                                          forward = False)



#!/usr/bin/env python
#
# w3c_bindings.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


from thug.DOM.W3C.Attr import Attr
from thug.DOM.W3C.CDATASection import CDATASection
from thug.DOM.W3C.CharacterData import CharacterData
from thug.DOM.W3C.Comment import Comment
from thug.DOM.W3C.DOMException import DOMException
from thug.DOM.W3C.DOMImplementation import DOMImplementation
from thug.DOM.W3C.Document import Document
from thug.DOM.W3C.DocumentFragment import DocumentFragment
from thug.DOM.W3C.DocumentType import DocumentType
from thug.DOM.W3C.Element import Element
from thug.DOM.W3C.NamedNodeMap import NamedNodeMap
from thug.DOM.W3C.Node import Node
from thug.DOM.W3C.NodeList import NodeList
from thug.DOM.W3C.ProcessingInstruction import ProcessingInstruction
from thug.DOM.W3C.Text import Text
from thug.DOM.W3C.HTML.HTMLAllCollection import HTMLAllCollection
from thug.DOM.W3C.HTML.HTMLAnchorElement import HTMLAnchorElement
from thug.DOM.W3C.HTML.HTMLAppletElement import HTMLAppletElement
from thug.DOM.W3C.HTML.HTMLBRElement import HTMLBRElement
from thug.DOM.W3C.HTML.HTMLBaseElement import HTMLBaseElement
from thug.DOM.W3C.HTML.HTMLBaseFontElement import HTMLBaseFontElement
from thug.DOM.W3C.HTML.HTMLBodyElement import HTMLBodyElement
from thug.DOM.W3C.HTML.HTMLButtonElement import HTMLButtonElement
from thug.DOM.W3C.HTML.HTMLCollection import HTMLCollection
from thug.DOM.W3C.HTML.HTMLDListElement import HTMLDListElement
from thug.DOM.W3C.HTML.HTMLDirectoryElement import HTMLDirectoryElement
from thug.DOM.W3C.HTML.HTMLDivElement import HTMLDivElement
from thug.DOM.W3C.HTML.HTMLDocument import HTMLDocument
from thug.DOM.W3C.HTML.HTMLDocumentCompatibleInfo import HTMLDocumentCompatibleInfo
from thug.DOM.W3C.HTML.HTMLElement import HTMLElement
from thug.DOM.W3C.HTML.HTMLFieldSetElement import HTMLFieldSetElement
from thug.DOM.W3C.HTML.HTMLFontElement import HTMLFontElement
from thug.DOM.W3C.HTML.HTMLFormElement import HTMLFormElement
from thug.DOM.W3C.HTML.HTMLFrameElement import HTMLFrameElement
from thug.DOM.W3C.HTML.HTMLFrameSetElement import HTMLFrameSetElement
from thug.DOM.W3C.HTML.HTMLHRElement import HTMLHRElement
from thug.DOM.W3C.HTML.HTMLHeadElement import HTMLHeadElement
from thug.DOM.W3C.HTML.HTMLHeadingElement import HTMLHeadingElement
from thug.DOM.W3C.HTML.HTMLHtmlElement import HTMLHtmlElement
from thug.DOM.W3C.HTML.HTMLIFrameElement import HTMLIFrameElement
from thug.DOM.W3C.HTML.HTMLImageElement import HTMLImageElement
from thug.DOM.W3C.HTML.HTMLInputElement import HTMLInputElement
from thug.DOM.W3C.HTML.HTMLIsIndexElement import HTMLIsIndexElement
from thug.DOM.W3C.HTML.HTMLLIElement import HTMLLIElement
from thug.DOM.W3C.HTML.HTMLLabelElement import HTMLLabelElement
from thug.DOM.W3C.HTML.HTMLLegendElement import HTMLLegendElement
from thug.DOM.W3C.HTML.HTMLLinkElement import HTMLLinkElement
from thug.DOM.W3C.HTML.HTMLMenuElement import HTMLMenuElement
from thug.DOM.W3C.HTML.HTMLMetaElement import HTMLMetaElement
from thug.DOM.W3C.HTML.HTMLModElement import HTMLModElement
from thug.DOM.W3C.HTML.HTMLOListElement import HTMLOListElement
from thug.DOM.W3C.HTML.HTMLObjectElement import HTMLObjectElement
from thug.DOM.W3C.HTML.HTMLOptGroupElement import HTMLOptGroupElement
from thug.DOM.W3C.HTML.HTMLOptionElement import HTMLOptionElement
from thug.DOM.W3C.HTML.HTMLOptionsCollection import HTMLOptionsCollection
from thug.DOM.W3C.HTML.HTMLParagraphElement import HTMLParagraphElement
from thug.DOM.W3C.HTML.HTMLParamElement import HTMLParamElement
from thug.DOM.W3C.HTML.HTMLPreElement import HTMLPreElement
from thug.DOM.W3C.HTML.HTMLQuoteElement import HTMLQuoteElement
from thug.DOM.W3C.HTML.HTMLScriptElement import HTMLScriptElement
from thug.DOM.W3C.HTML.HTMLSelectElement import HTMLSelectElement
from thug.DOM.W3C.HTML.HTMLSpanElement import HTMLSpanElement
from thug.DOM.W3C.HTML.HTMLStyleElement import HTMLStyleElement
from thug.DOM.W3C.HTML.HTMLTableCaptionElement import HTMLTableCaptionElement
from thug.DOM.W3C.HTML.HTMLTableCellElement import HTMLTableCellElement
from thug.DOM.W3C.HTML.HTMLTableColElement import HTMLTableColElement
from thug.DOM.W3C.HTML.HTMLTableElement import HTMLTableElement
from thug.DOM.W3C.HTML.HTMLTableRowElement import HTMLTableRowElement
from thug.DOM.W3C.HTML.HTMLTableSectionElement import HTMLTableSectionElement
from thug.DOM.W3C.HTML.HTMLTextAreaElement import HTMLTextAreaElement
from thug.DOM.W3C.HTML.HTMLTitleElement import HTMLTitleElement
from thug.DOM.W3C.HTML.HTMLUListElement import HTMLUListElement
from thug.DOM.W3C.Events.Event import Event
from thug.DOM.W3C.Events.EventTarget import EventTarget
from thug.DOM.W3C.Events.MouseEvent import MouseEvent
from thug.DOM.W3C.Events.MutationEvent import MutationEvent
from thug.DOM.W3C.Events.StorageEvent import StorageEvent
from thug.DOM.W3C.Events.UIEvent import UIEvent
from thug.DOM.W3C.Style.CSS.CSSStyleDeclaration import CSSStyleDeclaration


w3c_bindings = {
                'Attr'                       : Attr,
                'CDATASection'               : CDATASection,
                'CharacterData'              : CharacterData,
                'Comment'                    : Comment,
                'DOMException'               : DOMException,
                'DOMImplementation'          : DOMImplementation,
                'Document'                   : Document,
                'DocumentFragment'           : DocumentFragment,
                'DocumentType'               : DocumentType,
                'Element'                    : Element,
                'NamedNodeMap'               : NamedNodeMap,
                'Node'                       : Node,
                'NodeList'                   : NodeList,
                'ProcessingInstruction'      : ProcessingInstruction,
                'Text'                       : Text,
                'HTMLAllCollection'          : HTMLAllCollection,
                'HTMLAnchorElement'          : HTMLAnchorElement,
                'HTMLAppletElement'          : HTMLAppletElement,
                'HTMLBRElement'              : HTMLBRElement,
                'HTMLBaseElement'            : HTMLBaseElement,
                'HTMLBaseFontElement'        : HTMLBaseFontElement,
                'HTMLBodyElement'            : HTMLBodyElement,
                'HTMLButtonElement'          : HTMLButtonElement,
                'HTMLCollection'             : HTMLCollection,
                'HTMLDListElement'           : HTMLDListElement,
                'HTMLDirectoryElement'       : HTMLDirectoryElement,
                'HTMLDivElement'             : HTMLDivElement,
                'HTMLDocument'               : HTMLDocument,
                'HTMLDocumentCompatibleInfo' : HTMLDocumentCompatibleInfo,
                'HTMLElement'                : HTMLElement,
                'HTMLFieldSetElement'        : HTMLFieldSetElement,
                'HTMLFontElement'            : HTMLFontElement,
                'HTMLFormElement'            : HTMLFormElement,
                'HTMLFrameElement'           : HTMLFrameElement,
                'HTMLFrameSetElement'        : HTMLFrameSetElement,
                'HTMLHRElement'              : HTMLHRElement,
                'HTMLHeadElement'            : HTMLHeadElement,
                'HTMLHeadingElement'         : HTMLHeadingElement,
                'HTMLHtmlElement'            : HTMLHtmlElement,
                'HTMLIFrameElement'          : HTMLIFrameElement,
                'HTMLImageElement'           : HTMLImageElement,
                'HTMLInputElement'           : HTMLInputElement,
                'HTMLIsIndexElement'         : HTMLIsIndexElement,
                'HTMLLIElement'              : HTMLLIElement,
                'HTMLLabelElement'           : HTMLLabelElement,
                'HTMLLegendElement'          : HTMLLegendElement,
                'HTMLLinkElement'            : HTMLLinkElement,
                'HTMLMenuElement'            : HTMLMenuElement,
                'HTMLMetaElement'            : HTMLMetaElement,
                'HTMLModElement'             : HTMLModElement,
                'HTMLOListElement'           : HTMLOListElement,
                'HTMLObjectElement'          : HTMLObjectElement,
                'HTMLOptGroupElement'        : HTMLOptGroupElement,
                'HTMLOptionElement'          : HTMLOptionElement,
                'HTMLOptionsCollection'      : HTMLOptionsCollection,
                'HTMLParagraphElement'       : HTMLParagraphElement,
                'HTMLParamElement'           : HTMLParamElement,
                'HTMLPreElement'             : HTMLPreElement,
                'HTMLQuoteElement'           : HTMLQuoteElement,
                'HTMLScriptElement'          : HTMLScriptElement,
                'HTMLSelectElement'          : HTMLSelectElement,
                'HTMLSpanElement'            : HTMLSpanElement,
                'HTMLStyleElement'           : HTMLStyleElement,
                'HTMLTableCaptionElement'    : HTMLTableCaptionElement,
                'HTMLTableCellElement'       : HTMLTableCellElement,
                'HTMLTableColElement'        : HTMLTableColElement,
                'HTMLTableElement'           : HTMLTableElement,
                'HTMLTableRowElement'        : HTMLTableRowElement,
                'HTMLTableSectionElement'    : HTMLTableSectionElement,
                'HTMLTextAreaElement'        : HTMLTextAreaElement,
                'HTMLTitleElement'           : HTMLTitleElement,
                'HTMLUListElement'           : HTMLUListElement,
                'Event'                      : Event,
                'EventTarget'                : EventTarget,
                'MouseEvent'                 : MouseEvent,
                'MutationEvent'              : MutationEvent,
                'StorageEvent'               : StorageEvent,
                'UIEvent'                    : UIEvent,
                'CSSStyleDeclaration'        : CSSStyleDeclaration,
                }



#!/usr/bin/env python
#
# BaseLogging.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import errno
import hashlib
import datetime
import logging

log = logging.getLogger("Thug")


class BaseLogging(object):
    def __init__(self):
        pass

    def check_module(self, module, config):
        if not getattr(log.ThugOpts, "%s_logging" % (module, ), True):
            return False

        try:
            section = config.options(module)
        except:  # pylint:disable=bare-except
            return True

        for option in section:
            if option not in ('enable', ):
                continue

            enable = config.get(module, option)
            if enable.lower() in ('false', ):
                return False

        return True

    def set_basedir(self, url):
        if self.baseDir:
            return

        t = datetime.datetime.now()
        m = hashlib.md5()
        m.update(url.encode('utf8'))

        base = os.getenv('THUG_LOGBASE', os.pardir if os.access(os.pardir, os.W_OK) else '/tmp/thug')
        self.baseDir = os.path.join(base, 'logs', m.hexdigest(), t.strftime("%Y%m%d%H%M%S"))

        if not log.ThugOpts.file_logging:
            return

        try:
            os.makedirs(self.baseDir)
        except OSError as e:
            if e.errno == errno.EEXIST:
                pass
            else:
                raise

        thug_csv = os.path.join(base, 'logs', 'thug.csv')
        csv_line = '{},{}\n'.format(m.hexdigest(), url)

        if os.path.exists(thug_csv):
            with open(thug_csv, 'r') as fd:
                for l in fd.readlines():
                    if l == csv_line:
                        return

        with open(thug_csv, 'at+') as fd:
            fd.write(csv_line)

    def set_absbasedir(self, basedir):
        self.baseDir = basedir

        if not log.ThugOpts.file_logging:
            return

        try:
            os.makedirs(self.baseDir)
        except OSError as e:
            if e.errno == errno.EEXIST:
                pass
            else:
                raise



# CA BrightStor
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def AddColumn(self, arg0, arg1):
    if len(arg0) > 100:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "CA BrightStor ActiveX",
                                          "Overflow in AddColumn")



# DVRHOST Web CMS OCX 1.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def TimeSpanFormat(self, arg0, arg1):
    if len(arg1) > 512:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "DVRHOST Web CMS OCX ActiveX",
                                          "Overflow in TimeSpanFormat")



#!/usr/bin/env python


class DocumentCompatibleInfoCollection(object):
    """
    http://msdn.microsoft.com/en-us/library/hh826015(v=vs.85).aspx

    There are no standards that apply here.
    """

    def __init__(self, doc, nodes):
        self.doc   = doc
        self.nodes = nodes

    def __len__(self):
        return self.length

    def __getitem__(self, key):
        try:
            return self.item(int(key))
        except ValueError:
            return None

    @property
    def length(self):
        return len(self.nodes)

    @property
    def constructor(self):
        return None

    def item(self, index):
        if index > self.length - 1:
            return None

        return self.nodes[index]



#!/usr/bin/env python
#
# VirusTotal.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


import os
import json
import requests
import tempfile
import logging

log = logging.getLogger("Thug")

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser


class VirusTotal(object):
    def __init__(self):
        self.enabled = True
        self.opts    = dict()

        self.__init_config()

    def __init_config(self):
        config = ConfigParser.ConfigParser()

        conf_file = os.path.join(log.configuration_path, 'virustotal.conf')

        if not os.path.exists(conf_file):
            conf_file = os.path.join(log.configuration_path, 'virustotal.conf.default')

        if not os.path.isfile(conf_file):
            log.warning("[WARNING] VirusTotal disabled (no configuration file found)")
            self.enabled = False
            return

        config.read(conf_file)

        for option in config.options('VirusTotal'):
            self.opts[option] = config.get('VirusTotal', option)

        runtime_apikey = log.ThugOpts.get_vt_runtime_apikey()
        if runtime_apikey:
            self.opts['apikey'] = runtime_apikey

        if not self.opts.get('apikey', None):
            self.enabled = False

    def save_report(self, response_dict, basedir, sample):
        log_dir = os.path.join(basedir, 'analysis', 'virustotal')
        content = json.dumps(response_dict)

        log.ThugLogging.log_virustotal(log_dir, sample, content)

        positives = str(response_dict.get("positives", {}))
        total     = str(response_dict.get("total", {}))

        log.warning("[VirusTotal] Sample %s analysis ratio: %s/%s", response_dict['md5'], positives, total)

    def get_report(self, report):
        params   = { "resource": report,
                     "allinfo" : 1,
                     "apikey"  : self.opts['apikey']}

        response = requests.get(self.opts["reporturl"], params = params)
        return response

    def query(self, sample, basedir):
        md5           = sample['md5']
        response      = self.get_report(md5)
        response_dict = response.json()
        response_code = response_dict.get("response_code")

        if response.ok:
            if response_code == 1:
                self.save_report(response_dict, basedir, sample)
                return True

            log.warning("[VirusTotal] %s", response_dict['verbose_msg'])

        return False

    def submit(self, data, sample):
        md5   = sample['md5']
        fd, s = tempfile.mkstemp()

        with open(s, "wb") as fd:
            fd.write(data)

        params   = {'apikey': self.opts['apikey']}
        files    = {'file'  : (md5, open(s, "rb"))}
        response = requests.post(self.opts["scanurl"], files = files, params = params)

        if response.ok:
            log.warning("[VirusTotal] Sample %s submitted", md5)

        os.remove(s)

    def analyze(self, data, sample, basedir):
        if not self.enabled:
            return

        if not self.opts['apikey']:
            return

        if sample.get('md5', None) and log.ThugOpts.vt_query and self.query(sample, basedir):
            return

        if log.ThugOpts.vt_submit:
            self.submit(data, sample)



# SonicWall SSL-VPN NetExtender NELaunchCtrl ActiveX control
# CVE-2007-5603 (AddRouteEntry)

import logging

log = logging.getLogger("Thug")


def AddRouteEntry(self, arg0, arg1):
    if len(arg0) > 20 or len(arg1) > 20:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "SonicWall SSL-VPN NetExtender NELaunchCtrl ActiveX",
                                          "Overflow in AddRouteEntry",
                                          cve = 'CVE-2007-5603')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5603", None)



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLTableCaptionElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    align           = attr_property("align")



#!/usr/bin/env python
#
# ElasticSearch.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA
#

import os
import logging

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser

from .JSON import JSON

log = logging.getLogger("Thug")

try:
    from elasticsearch import Elasticsearch, RequestsHttpConnection
    ELASTICSEARCH_MODULE = True
except ImportError:
    ELASTICSEARCH_MODULE = False


class ElasticSearch(JSON):
    def __init__(self, thug_version):
        JSON.__init__(self, thug_version, provider = True)

        self.enabled = True

        if not ELASTICSEARCH_MODULE:
            self.enabled = False
            return

        if not log.ThugOpts.elasticsearch_logging:
            self.enabled = False
            return

        if not self.__init_elasticsearch():
            self.enabled = False
            return

    def __init_config(self):
        self.opts = dict()

        config = ConfigParser.ConfigParser()

        conf_file = os.path.join(log.configuration_path, 'logging.conf')

        if not os.path.exists(conf_file):
            conf_file = os.path.join(log.configuration_path, 'logging.conf.default')

        if not os.path.exists(conf_file):
            return False

        config.read(conf_file)

        for option in config.options('elasticsearch'):
            self.opts[option] = config.get('elasticsearch', option)

        if self.opts['enable'].lower() in ('false', ):
            return False

        return True

    def __init_elasticsearch(self):
        if not self.__init_config():
            return False

        self.es = Elasticsearch(self.opts['url'], connection_class = RequestsHttpConnection)

        if not self.es.ping():
            log.warning("[WARNING] ElasticSearch instance not properly initialized")
            return False

        self.es.indices.create(index = self.opts['index'], ignore = 400)  # pylint:disable=unexpected-keyword-arg
        return True

    def export(self, basedir):
        if not self.enabled:
            return

        res = self.es.index(index = self.opts['index'], doc_type = "analysis", body = self.data)
        return res['created']



#!/usr/bin/env python
#
# ThugLogging.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .BaseLogging import BaseLogging
from .SampleLogging import SampleLogging
from .LoggingModules import LoggingModules
from thug.Analysis.virustotal.VirusTotal import VirusTotal
from thug.Analysis.honeyagent.HoneyAgent import HoneyAgent

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser

import os
import errno
import copy

import logging

log = logging.getLogger("Thug")


class ThugLogging(BaseLogging, SampleLogging):
    eval_min_length_logging = 4

    def __init__(self, thug_version):
        BaseLogging.__init__(self)
        SampleLogging.__init__(self)

        self.thug_version   = thug_version
        self.VirusTotal     = VirusTotal()
        self.HoneyAgent     = HoneyAgent()
        self.baseDir        = None
        self.windows        = dict()
        self.shellcodes     = set()
        self.shellcode_urls = set()
        self.methods_cache  = dict()
        self.formats        = set()
        self.url            = ""

        self.__init_config()

    def __init_config(self):
        self.modules = dict()
        config       = ConfigParser.ConfigParser()

        conf_file = os.path.join(log.configuration_path, 'logging.conf')

        if not os.path.exists(conf_file):
            conf_file = os.path.join(log.configuration_path, 'logging.conf.default')

        if not os.path.exists(conf_file):
            log.warning("[CRITICAL] Logging subsystem not initialized (configuration file not found)")
            return

        config.read(conf_file)

        for name, module in LoggingModules.items():
            if self.check_module(name, config):
                self.modules[name.strip()] = module(self.thug_version)

        for m in self.modules.values():
            for fmt in getattr(m, 'formats', tuple()):
                self.formats.add(fmt)

    def resolve_method(self, name):
        if name in self.methods_cache.keys():
            return self.methods_cache[name]

        methods = []

        for module in self.modules.values():
            m = getattr(module, name, None)
            if m:
                methods.append(m)

        self.methods_cache[name] = methods
        return methods

    def set_url(self, url):
        self.url = url

        for m in self.resolve_method('set_url'):
            m(url.encode('utf8'))

    def add_behavior_warn(self, description = None, cve = None, method = "Dynamic Analysis"):
        for m in self.resolve_method('add_behavior_warn'):
            m(description, cve, method)

        log.warning(description)

    def check_snippet(self, s):
        return len(s) < self.eval_min_length_logging

    def add_code_snippet(self, snippet, language, relationship, method = "Dynamic Analysis", check = False):
        if not log.ThugOpts.code_logging:
            return

        if check and self.check_snippet(snippet):
            return

        for m in self.resolve_method('add_code_snippet'):
            m(snippet, language, relationship, method)

    def add_shellcode_snippet(self, snippet, language, relationship, method):
        for m in self.resolve_method('add_code_snippet'):
            m(snippet, language, relationship, method)

    def log_file(self, data, url = None, params = None, sampletype = None):
        sample = self.build_sample(data, url, sampletype)
        if sample is None:
            return None

        return self.__log_file(sample, data, url, params)

    def __log_file(self, sample, data, url = None, params = None):
        for m in self.resolve_method('log_file'):
            m(copy.deepcopy(sample), url, params)

        self.VirusTotal.analyze(data, sample, self.baseDir)

        if sample['type'] in ('JAR', ):
            self.HoneyAgent.analyze(data, sample, self.baseDir, params)

        log.SampleClassifier.classify(data, sample['md5'])
        return sample

    def log_event(self):
        for m in self.resolve_method('export'):
            m(self.baseDir)

        for m in self.resolve_method('log_event'):
            m(self.baseDir)

        if log.ThugOpts.file_logging:
            log.warning("Thug analysis logs saved at %s", self.baseDir)

    def log_connection(self, source, destination, method, flags = None):
        """
        Log the connection (redirection, link) between two pages

        @source         The origin page
        @destination    The page the user is made to load next
        @method         Link, iframe, .... that moves the user from source to destination
        @flags          Additional information flags. Existing are: "exploit"
        """
        if flags is None:
            flags = dict()

        for m in self.resolve_method('log_connection'):
            m(source, destination, method, flags)

    def log_location(self, url, data, flags = None):
        """
        Log file information for a given url

        @url    URL we fetched this file from
        @data   File dictionary data
                    Keys:
                        - content     Content
                        - md5         MD5 checksum
                        - sha256      SHA-256 checksum
                        - fsize       Content size
                        - ctype       Content type (whatever the server says it is)
                        - mtype       Calculated MIME type

        @flags  Additional information flags
        """
        if flags is None:
            flags = dict()

        for m in self.resolve_method('log_location'):
            m(url, data, flags = flags)

    def log_exploit_event(self, url, module, description, cve = None, data = None, forward = True):
        """
        Log file information for a given url

        @url            URL where this exploit occured
        @module         Module/ActiveX Control, ... that gets exploited
        @description    Description of the exploit
        @cve            CVE number (if available)
        @forward        Forward log to add_behavior_warn
        """
        if forward:
            self.add_behavior_warn("[%s] %s" % (module, description, ), cve = cve)

        for m in self.resolve_method('log_exploit_event'):
            m(url, module, description, cve = cve, data = data)

    def log_classifier(self, classifier, url, rule, tags):
        """
        Log classifiers matching for a given url

        @classifier     Classifier name
        @url            URL where the rule match occurred
        @rule           Rule name
        @tags           Rule tags
        """
        self.add_behavior_warn("[%s Classifier] URL: %s (Rule: %s, Classification: %s)" % (classifier.upper(), url, rule, tags, ))

        for m in self.resolve_method('log_classifier'):
            m(classifier, url, rule, tags)

    def log_warning(self, data):
        #log.warning(data)

        for m in self.resolve_method('log_warning'):
            m(data)

    def log_redirect(self, response):
        if not response:
            return None

        if not response.history:
            if response.url:
                log.URLClassifier.classify(response.url)
                log.HTTPSession.fetch_ssl_certificate(response.url)

            return None

        final = response.url

        while final is None:
            for h in reversed(response.history):
                final = h.url

        for h in response.history:
            location = h.headers.get('location', None)

            self.add_behavior_warn("[HTTP Redirection (Status: %s)] Content-Location: %s --> Location: %s" % (h.status_code,
                                                                                                              h.url,
                                                                                                              location))
            self.log_connection(h.url, location, "http-redirect")

            log.URLClassifier.classify(h.url)
            log.HTTPSession.fetch_ssl_certificate(h.url)

        log.URLClassifier.classify(final)
        log.HTTPSession.fetch_ssl_certificate(final)

        return final

    def log_href_redirect(self, referer, url):
        self.add_behavior_warn("[HREF Redirection (document.location)] Content-Location: %s --> Location: %s" % (referer, url, ))
        self.log_connection(referer, url, "href")

    def log_certificate(self, url, certificate):
        if not log.ThugOpts.cert_logging:
            return

        self.add_behavior_warn("[Certificate]\n %s" % (certificate, ))

        for m in self.resolve_method('log_certificate'):
            m(url, certificate)

    def log_analysis_module(self, dirname, sample, report, module, fmt = "json"):
        filename = "%s.%s" % (sample['md5'], fmt, )
        self.store_content(dirname, filename, report)

        method = "log_%s" % (module, )
        for m in self.resolve_method(method):
            m(sample, report)

    def log_virustotal(self, dirname, sample, report):
        self.log_analysis_module(dirname, sample, report, "virustotal")

    def log_honeyagent(self, dirname, sample, report):
        self.log_analysis_module(dirname, sample, report, "honeyagent")

    def log_androguard(self, dirname, sample, report):
        self.__log_file(sample, sample.pop("raw", None))
        self.log_analysis_module(dirname, sample, report, "androguard", "txt")

    def log_peepdf(self, dirname, sample, report):
        self.log_analysis_module(dirname, sample, report, "peepdf", "xml")

    def store_content(self, dirname, filename, content):
        """
        This method is meant to be used when a content (downloaded
        pages, samples, reports, etc. ) has to be saved in a flat
        file.

        @dirname    The directory where to store content
        @filename   The file where to store content
        @content    The content to be stored
        """
        if not log.ThugOpts.file_logging:
            return

        try:
            os.makedirs(dirname)
        except OSError as e:
            if e.errno == errno.EEXIST:
                pass
            else:
                raise

        fname = os.path.join(dirname, filename)

        with open(fname, 'wb') as fd:
            fd.write(content)

        return fname



#!/usr/bin/env python

from .DOMException import DOMException
from .Node import Node


class CharacterData(Node):
    def __init__(self, doc, data):
        self._data = data
        Node.__init__(self, doc)

    def __str__(self):
        return str(self.data)

    def getData(self):
        return self._data

    def setData(self, data):
        raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)

    data = property(getData, setData)

    @property
    def length(self):
        return len(self.data)

    def substringData(self, offset, count):
        return self.data[offset:offset + count]

    def appendData(self, arg):
        raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)

    def insertData(self, offset, arg):
        raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)

    def deleteData(self, offset, count):
        raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)

    def replaceData(self, offset, count, arg):
        raise DOMException(DOMException.NO_MODIFICATION_ALLOWED_ERR)



#!/usr/bin/env python

from .Event import Event
from .HTMLEvent import HTMLEvent
from .MouseEvent import MouseEvent

import logging
log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class EventTarget(object):
    def __init__(self):
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 9:
            self.detachEvent = self._detachEvent

            def attachEvent(self, eventType, handler, prio = False):
                return self._attachEvent(eventType, handler, prio)

            setattr(self.__class__, 'attachEvent', attachEvent)
        else:
            self.removeEventListener = self._removeEventListener

            def addEventListener(self, eventType, listener, capture = False):
                return self._addEventListener(eventType, listener, capture)

            setattr(self.__class__, 'addEventListener', addEventListener)

    def __insert_listener(self, eventType, listener, capture, prio):
        # A document element or other object may have more than one event
        # handler registered for a particular type of event. When an appropriate
        # event occurs, the browser must invoke all of the handlers, following
        # these rules of invocation order:
        #
        # - Handlers registered by setting an object property or HTML attribute,
        # if any, are always invoked first
        #
        # - Handlers registered with addEventListener() are invoked in the order
        # in which they were registered
        #
        # - Handlers registered with attachEvent() may be invoked in any order
        # and your code should not depend on sequential invocation
        #
        # The `prio' parameter is used for deciding if the handler has to be
        # appended at the end of the _listener list (addEventListener and
        # attachEvent) or at the beginning (setting an object property or HTML
        # attribute)
        if prio:
            self.tag._listeners.insert(0, (eventType, listener, capture))
        else:
            self.tag._listeners.append((eventType, listener, capture))

    def _addEventListener(self, eventType, listener, capture = False, prio = False):
        log.debug('_addEventListener(%s, \n%r, \n%s)', eventType, listener, capture)

        if getattr(self.tag, '_listeners', None) is None:
            self.tag._listeners = list()

        if not (eventType, listener, capture) in self.tag._listeners:
            self.__insert_listener(eventType, listener, capture, prio)
            return

        # attachEvent() allows the same event handler to be registered more than
        # once. When the event of the specified type occurs, the registered
        # function will be invoked as many times as it was registered
        if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 9:
            self.__insert_listener(eventType, listener, capture, prio)

    def _removeEventListener(self, eventType, listener, capture = False):
        log.debug('_removeEventListener(%s, \n%r, \n%s)', eventType, listener, capture)

        try:
            self.tag._listeners.remove((eventType, listener, capture))
        except:  # pylint:disable=bare-except
            pass

    def _attachEvent(self, eventType, handler, prio = False):
        log.debug('_attachEvent(%s, \n%r)', eventType, handler)
        if not eventType.startswith('on'):
            log.warning('[WARNING] attachEvent eventType: %s', eventType)

        self._addEventListener(eventType[2:], handler, False, prio)

    def _detachEvent(self, eventType, handler):
        log.debug('_detachEvent(%s, \n%r)', eventType, handler)
        if not eventType.startswith('on'):
            log.warning('[WARNING] detachEvent eventType: %s', eventType)

        self._removeEventListener(eventType[2:], handler)

    def _get_listeners(self, tag, evtType):
        _listeners         = [(eventType, listener, capture) for (eventType, listener, capture) in tag._listeners if eventType == evtType]
        capture_listeners  = [(eventType, listener, capture) for (eventType, listener, capture) in _listeners if capture is True]
        bubbling_listeners = [(eventType, listener, capture) for (eventType, listener, capture) in _listeners if capture is False]
        return capture_listeners, bubbling_listeners

    def _do_dispatch(self, c, evtObject):
        eventType, listener, capture = c  # pylint:disable=unused-variable

        with self.doc.window.context as ctx:  # pylint:disable=unused-variable
            if log.ThugOpts.Personality.isIE() and log.ThugOpts.Personality.browserMajorVersion < 9:
                self.doc.window.event = evtObject
                listener()
            else:
                listener(evtObject)

    def do_dispatch(self, c, evtObject):
        try:
            self._do_dispatch(c, evtObject)
        except:  # pylint:disable=bare-except
            eventType, listener, capture = c  # pylint:disable=unused-variable
            log.warning("[WARNING] Error while dispatching %s event", eventType)

    def _dispatchCaptureEvent(self, tag, evtType, evtObject):
        if tag.parent is None:
            return

        self._dispatchCaptureEvent(tag.parent, evtType, evtObject)

        if not tag.parent._listeners:
            return

        if evtObject._stoppedPropagation:
            return

        capture_listeners, bubbling_listeners = self._get_listeners(tag.parent, evtType)  # pylint:disable=unused-variable
        for c in capture_listeners:
            evtObject.currentTarget = tag.parent._node
            self.do_dispatch(c, evtObject)

    def _dispatchBubblingEvent(self, tag, evtType, evtObject):
        for node in tag.parents:
            if node is None:
                break

            if not node._listeners:
                continue

            if evtObject._stoppedPropagation:
                continue

            capture_listeners, bubbling_listeners = self._get_listeners(node, evtType)  # pylint:disable=unused-variable
            for c in bubbling_listeners:
                evtObject.currentTarget = node._node
                self.do_dispatch(c, evtObject)

    def dispatchEvent(self, evtType):
        log.info('dispatchEvent(%s)', evtType)
        evtObject = None

        if evtType in MouseEvent.MouseEventTypes:
            evtObject = MouseEvent(evtType, self)

        if evtType in HTMLEvent.HTMLEventTypes:
            evtObject = HTMLEvent(evtType, self)

        # print evtObject
        capture_listeners, bubbling_listeners = self._get_listeners(self.tag, evtType)

        if capture_listeners:
            evtObject.eventPhase = Event.CAPTURING_PHASE
            self._dispatchCaptureEvent(self.tag, evtType, evtObject)

        evtObject.eventPhase    = Event.AT_TARGET
        evtObject.currentTarget = self

        if not evtObject._stoppedPropagation:
            for c in capture_listeners:
                self.do_dispatch(c, evtObject)

            for c in bubbling_listeners:
                self.do_dispatch(c, evtObject)

        if bubbling_listeners:
            evtObject.eventPhase = Event.BUBBLING_PHASE
            self._dispatchBubblingEvent(self.tag, evtType, evtObject)

        evtObject.eventPhase = Event.AT_TARGET
        return True




import logging

log = logging.getLogger("Thug")


def LaunchGui(self, arg0, arg1, arg2):
    if len(arg0) > 1500:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "EnjoySAP ActiveX",
                                          "LaunchGUI overflow in arg0")


def PrepareToPostHTML(self, arg):
    if len(arg) > 1000:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "EnjoySAP ActiveX",
                                          "PrepareToPostHTM overflow in arg0")


def Comp_Download(self, arg0, arg1):
    log.warning(arg0)
    log.warning(arg1)

    url = arg0

    log.ThugLogging.add_behavior_warn("[EnjoySAP ActiveX] Fetching from URL %s" % (url, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "EnjoySAP ActiveX",
                                      "Fetching from URL",
                                      data = {
                                                "url" : url
                                             },
                                      forward = False)

    try:
        self._window._navigator.fetch(url, redirect_type = "EnjoySAP Exploit")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[EnjoySAP ActiveX] Fetch failed')




import logging
log = logging.getLogger("Thug")


def LinkSBIcons(self):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "AOL ActiveX",
                                      "Attack in LinkSBIcons function",
                                      cve = "CVE-2006-5820")

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2006-5820", None)



# Persists Software XUpload control, version 2.1.0.1.
# CVE-2007-6530

import logging

log = logging.getLogger("Thug")


def AddFolder(self, arg):
    if len(arg) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "XUpload ActiveX",
                                          "Overflow in AddFolder method",
                                          cve = 'CVE-2007-6530')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6530", None)

def AddFile(self, arg):
    if len(arg) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "XUpload ActiveX",
                                          "Overflow in AddFile method",
                                          cve = 'CVE-2007-6530')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-6530", None)



#!/usr/bin/env python
#
# ThugVulnModules.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
log = logging.getLogger("Thug")

class ThugVulnModules(dict):
    def __init__(self):
        self._acropdf_pdf               = '9.1.0'
        self._acropdf_disabled          = False
        self._shockwave_flash           = '10.0.64.0'
        self._shockwave_flash_disabled  = False
        self._javaplugin                = '1.6.0.32'
        self._javaplugin_disabled       = False

    def invalid_version(self, version):
        for p in version.split('.'):
            if not p.isdigit():
                return True

        return False

    def get_acropdf_pdf(self):
        return self._acropdf_pdf

    @property
    def acropdf(self):
        return self._acropdf_pdf

    def set_acropdf_pdf(self, version):
        if self.invalid_version(version):
            log.warning('[WARNING] Invalid Adobe Acrobat Reader version provided (using default one)')
            return

        self._acropdf_pdf = version

    acropdf_pdf = property(get_acropdf_pdf, set_acropdf_pdf)

    def disable_acropdf(self):
        self._acropdf_disabled = True

    @property
    def acropdf_disabled(self):
        return self._acropdf_disabled

    def get_shockwave_flash(self):
        return self._shockwave_flash

    def set_shockwave_flash(self, version):
        if not version.split('.')[0] in ('8', '9', '10', '11', '12', ) or self.invalid_version(version):
            log.warning('[WARNING] Invalid Shockwave Flash version provided (using default one)')
            return

        self._shockwave_flash = version

    shockwave_flash = property(get_shockwave_flash, set_shockwave_flash)

    def disable_shockwave_flash(self):
        self._shockwave_flash_disabled = True

    @property
    def shockwave_flash_disabled(self):
        return self._shockwave_flash_disabled

    def get_javaplugin(self):
        javaplugin = self._javaplugin.split('.')
        last       = javaplugin.pop()
        return '%s_%s' % (''.join(javaplugin), last)

    def set_javaplugin(self, version):
        if self.invalid_version(version):
            log.warning('[WARNING] Invalid JavaPlugin version provided (using default one)')
            return

        _version = version.split('.')
        while len(_version) < 4:
            _version.append('0')

        if len(_version[3]) == 1:
            _version[3] = '0%s' % (_version[3], )

        self._javaplugin = '.'.join(_version)

    javaplugin = property(get_javaplugin, set_javaplugin)

    def disable_javaplugin(self):
        self._javaplugin_disabled = True

    @property
    def javaplugin_disabled(self):
        return self._javaplugin_disabled

    @property
    def javawebstart_isinstalled(self):
        javawebstart = self._javaplugin.split('.')
        last         = javawebstart.pop() #pylint:disable=unused-variable
        return '%s.%s' % ('.'.join(javawebstart), '0')



#!/usr/bin/env python
#
# WebTracking.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .Cookies import Cookies
from .WebStorage import WebStorage

import logging
log = logging.getLogger("Thug")


class WebTracking(object):
    def __init__(self):
        self.cookies    = Cookies()
        self.webstorage = WebStorage()

    def inspect_response(self, response):
        if not log.ThugOpts.web_tracking:
            return

        self.cookies.inspect(response)

    def inspect_storage_setitem(self, storage, key, value):
        if not log.ThugOpts.web_tracking:
            return

        self.webstorage.inspect_set_item(storage, key, value)

    def inspect_storage_removeitem(self, storage, key):
        if not log.ThugOpts.web_tracking:
            return

        self.webstorage.inspect_remove_item(storage, key)

    def inspect_storage_clear(self, storage):
        if not log.ThugOpts.web_tracking:
            return

        self.webstorage.inspect_clear(storage)



import collections

# PyV8 code


class JSClass(object):
    __properties__  = {}
    __watchpoints__ = {}

    def __str__(self):
        return self.toString()

    def __unicode__(self):
        return self.toString()

    def __getattr__(self, name):
        if name == 'constructor':
            return JSClassConstructor(self.__class__)

        if name == 'prototype':
            return JSClassPrototype(self.__class__)

        prop = self.__dict__.setdefault('__properties__', {}).get(name, None)

        if prop and isinstance(prop[0], collections.Callable):
            return prop[0]()

        raise AttributeError(name)

    def __setattr__(self, name, value):
        prop = self.__dict__.setdefault('__properties__', {}).get(name, None)

        if prop and isinstance(prop[1], collections.Callable):
            return prop[1](value)

        return object.__setattr__(self, name, value)

    def toString(self):
        "Returns a string representation of an object."
        return "[object %s]" % self.__class__.__name__

    def toLocaleString(self):
        "Returns a value as a string value appropriate to the host environment's current locale."
        return self.toString()

    def valueOf(self):
        "Returns the primitive value of the specified object."
        return self

    def hasOwnProperty(self, name):
        "Returns a Boolean value indicating whether an object has a property with the specified name."
        return hasattr(self, name)

    def isPrototypeOf(self, obj):
        "Returns a Boolean value indicating whether an object exists in the prototype chain of another object."
        raise NotImplementedError()

    def __defineGetter__(self, name, getter):
        "Binds an object's property to a function to be called when that property is looked up."
        self.__properties__[name] = (getter, self.__lookupSetter__(name))

    def __lookupGetter__(self, name):
        "Return the function bound as a getter to the specified property."
        return self.__properties__.get(name, (None, None))[0]

    def __defineSetter__(self, name, setter):
        "Binds an object's property to a function to be called when an attempt is made to set that property."
        self.__properties__[name] = (self.__lookupGetter__(name), setter)

    def __lookupSetter__(self, name):
        "Return the function bound as a setter to the specified property."
        return self.__properties__.get(name, (None, None))[1]

    def watch(self, prop, handler):
        "Watches for a property to be assigned a value and runs a function when that occurs."
        self.__watchpoints__[prop] = handler

    def unwatch(self, prop):
        "Removes a watchpoint set with the watch method."
        del self.__watchpoints__[prop]


class JSClassConstructor(JSClass):
    def __init__(self, cls):
        self.cls = cls

    @property
    def name(self):
        return self.cls.__name__

    def toString(self):
        return "function %s() {\n  [native code]\n}" % self.name

    def __call__(self, *args, **kwds):
        return self.cls(*args, **kwds)


class JSClassPrototype(JSClass):
    def __init__(self, cls):
        self.cls = cls

    @property
    def constructor(self):
        return JSClassConstructor(self.cls)

    @property
    def name(self):
        return self.cls.__name__



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long


class HTMLTableColElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    align       = attr_property("align")
    ch          = attr_property("char")
    chOff       = attr_property("charoff")
    span        = attr_property("span", thug_long)
    vAlign      = attr_property("valign")
    width       = attr_property("width")



#!/usr/bin/env python
from thug.DOM.W3C.DOMException import DOMException


# Introduced in DOM Level 2
class DocumentEvent(object):
    def __init__(self, doc):
        self.doc = doc

    def createEvent(self, eventType):
        raise DOMException(DOMException.NOT_SUPPORTED_ERR)



# Sina DLoader Class ActiveX Control 'DonwloadAndInstall'
# Method Arbitrary File Download Vulnerability

import logging

log = logging.getLogger("Thug")


def DownloadAndInstall(self, url):
    log.ThugLogging.add_behavior_warn("[SinaDLoader Downloader ActiveX] Fetching from URL %s" % (url, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "SinaDLoader Downloader ActiveX",
                                      "Fetching from URL",
                                      data = {
                                                "url": url
                                             },
                                      forward = False)

    try:
        self._window._navigator.fetch(url, redirect_type = "SinaDLoader Exploit")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[SinaDLoader Downloader ActiveX] Fetch failed')



# Multiple Office OCX ActiveX Controls 'OpenWebFile()' Arbitrary
# Program Execution Vulnerability
# BID-33243

import logging

log = logging.getLogger("Thug")


def OpenWebFile(self, _file):
    log.ThugLogging.add_behavior_warn('[Office OCX ActiveX] OpenWebFile Arbitrary Program Execution Vulnerability')
    log.ThugLogging.add_behavior_warn("[Office OCX ActiveX] Fetching from URL %s" % (_file, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Office OCX ActiveX",
                                      "OpenWebFile Arbitrary Program Execution Vulnerability (BID-33243)",
                                      forward = False,
                                      data = {
                                                "url" : _file
                                             }
                                     )

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "BID-33243", None)

    try:
        self._window._navigator.fetch(_file, redirect_type = "Office OCX Exploit")
    except:  # pylint:disable=bare-except
        log.ThugLogging.add_behavior_warn('[Office OCX ActiveX] Fetch failed')



#!/usr/bin/env python
#
# Target.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

TARGET = """
    esprima.parse(%r, {

        // attach range information to each node
        //range: true,

        // attach line/column location information to each node
        //loc: true,

        // create a top-level comments array containing all comments
        //comments: true,

        // attach comments to the closest relevant node as leadingComments and
        // trailingComments
        //attachComment: true,

        // create a top-level tokens array containing all tokens
        // tokens: true,

        // try to continue parsing if an error is encountered, store errors in a
        // top-level errors array
        tolerant: true,

        // specify parsing features (default only has blockBindings: true)
        // setting this option replaces the default values
        ecmaFeatures: {

            // enable parsing of arrow functions
            arrowFunctions: true,

            // enable parsing of let/const
            blockBindings: true,

            // enable parsing of destructured arrays and objects
            destructuring: true,

            // enable parsing of regular expression y flag
            regexYFlag: true,

            // enable parsing of regular expression u flag
            regexUFlag: true,

            // enable parsing of template strings
            templateStrings: true,

            // enable parsing of binary literals
            binaryLiterals: true,

            // enable parsing of ES6 octal literals
            octalLiterals: true,

            // enable parsing unicode code point escape sequences
            unicodeCodePointEscapes: true,

            // enable parsing of default parameters
            defaultParams: true,

            // enable parsing of rest parameters
            restParams: true,

            // enable parsing of for-of statement
            forOf: true,

            // enable parsing computed object literal properties
            objectLiteralComputedProperties: true,

            // enable parsing of shorthand object literal methods
            objectLiteralShorthandMethods: true,

            // enable parsing of shorthand object literal properties
            objectLiteralShorthandProperties: true,

            // Allow duplicate object literal properties (except '__proto__')
            objectLiteralDuplicateProperties: true,

            // enable parsing of generators/yield
            generators: true,

            // enable parsing spread operator
            spread: true,

            // enable super in functions
            superInFunctions: true,

            // enable parsing classes
            classes: true,

            // enable parsing of new.target
            newTarget: false,

            // enable parsing of modules
            modules: true,

            // enable React JSX parsing
            jsx: true,

            // enable return in global scope
            globalReturn: true,

            // allow experimental object rest/spread
            //experimentalObjectRestSpread: true
        }
});
"""



#!/usr/bin/env python

import logging
from thug.DOM.W3C.DOMException import DOMException
from .HTMLElement import HTMLElement
from .HTMLCollection import HTMLCollection
from .HTMLTableCellElement import HTMLTableCellElement
from .attr_property import attr_property

log = logging.getLogger("Thug")


class HTMLTableRowElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)
        self._cells = HTMLCollection(doc, list())

    # Modified in DOM Level 2
    @property
    def rowIndex(self):
        raise NotImplementedError()

    # Modified in DOM Level 2
    @property
    def sectionRowIndex(self):
        raise NotImplementedError()

    # Modified in DOM Level 2
    @property
    def cells(self):
        return self._cells

    align           = attr_property("align")
    bgColor         = attr_property("bgcolor")
    ch              = attr_property("char")
    chOff           = attr_property("charoff")
    vAlign          = attr_property("valign")

    # Modified in DOM Level 2
    def insertCell(self, index = None):
        # `index' specifies the position of the row to insert (starts at 0).
        # The value of -1 can also be used; which result in that the new row
        # will be inserted at the last position. This parameter is required
        # in Firefox and Opera, but optional in Internet Explorer, Chrome and
        # Safari. If this parameter is omitted, insertRow() inserts a new row
        # at the last position in IE and at the first position in Chrome and
        # Safari.
        if index is None:
            if log.ThugOpts.Personality.isIE():
                index = -1
            if log.ThugOpts.Personality.isChrome() or log.ThugOpts.Personality.isSafari():
                index = 0

        cell = HTMLTableCellElement(self.doc, self.tag, index)

        if index in (-1, len(self._cells), ):
            self.cells.nodes.append(cell)
        else:
            self.cells.nodes.insert(index, cell)

        return cell

    # Modified in DOM Level 2
    def deleteCell(self, index):
        if index < -1 or index >= len(self.cells.nodes):
            raise DOMException(DOMException.INDEX_SIZE_ERR)

        del self.cells.nodes[index]



# Rising Online Virus Scanner Web Scan ActiveX Control
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def UpdateEngine(self):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Rising Online Virus Scanner Web Scan ActiveX",
                                      "UpdateEngine Method vulnerability")



# SSReader Pdg2 ActiveX control (pdg2.dll)
# CVE-2007-5892

import logging

log = logging.getLogger("Thug")


def Register(self, arg0, arg1):
    if len(arg1) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "SSReader Pdg2 ActiveX",
                                          "Register Method Overflow",
                                          cve = "CVE-2007-5892")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5892", None)


def LoadPage(self, arg0, arg1, arg2, arg3):
    if len(arg0) > 255:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "SSReader Pdg2 ActiveX",
                                          "LoadPage Method Overflow",
                                          cve = "CVE-2007-5892")

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2007-5892", None)



# Gateway Weblaunch ActiveX Control
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def DoWebLaunch(self, arg0, arg1, arg2, arg3):
    if len(arg1) > 512 or len(arg3) > 512:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Gateway Weblaunch ActiveX",
                                          "Overflow")
    else:
        log.ThugLogging.add_behavior_warn('[Gateway Weblaunch ActiveX] Trying to execute ' + arg1 + ' ' + arg2 + ' ' + arg3)



#!/usr/bin/env python

from .Node import Node
from .CharacterData import CharacterData


class Comment(CharacterData):
    @property
    def nodeName(self):
        return "#comment"

    @property
    def nodeType(self):
        return Node.COMMENT_NODE

    def getNodeValue(self):
        return self.data

    def setNodeValue(self, data):
        self.data = data

    nodeValue = property(getNodeValue, setNodeValue)



from io import BytesIO

import logging
from thug.Magic.Magic import Magic

log = logging.getLogger("Thug")

@property
def Size(self):
    fobject = getattr(self, 'fobject', None)
    if fobject is None:
        return 0

    content = self.fobject.getvalue()
    return len(content)


def open(self):  # pylint:disable=redefined-builtin
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] open")
    self.fobject = BytesIO()


def Read(self, length = -1):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] Read")

    fobject = getattr(self, 'fobject', None)
    if fobject is None:
        return None

    content = self.fobject.getvalue()

    if length > 0:
        length = min(length, len(content))

    return content[:length]


def Write(self, s):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] Write")
    self.fobject.write(s)


def SaveToFile(self, filename, opt = 0):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] SaveToFile(%s, %s)" % (filename, opt, ))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Adodb.Stream ActiveX",
                                      "SaveToFile",
                                      data = {
                                                "file": filename
                                             },
                                      forward = False)

    content = self.fobject.getvalue()
    mtype   = Magic(content).get_mime()

    log.ThugLogging.log_file(content, url = filename, sampletype = mtype)
    self._files[filename] = content


def LoadFromFile(self, filename):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] LoadFromFile(%s)" % (filename, ))
    if filename not in self._files:
        raise TypeError()

    self._current = filename


def ReadText(self, NumChars = -1):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] ReadText")

    if NumChars == -1:
        return self._files[self._current]

    return self._files[self._current][:NumChars - 1]


def WriteText(self, data, options = None):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] WriteText(%s)" % (data, ))


def Close(self):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] Close")
    self.fobject.close()


def setPosition(self, pos):
    log.ThugLogging.add_behavior_warn("[Adodb.Stream ActiveX] Changed position in fileobject to: (%s)" % (pos, ))
    self.__dict__['position'] = pos
    self.fobject.seek(pos)



#!/usr/bin/env python
#
# ThugPlugins.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import sys
import logging
from zope.interface.verify import verifyObject
from zope.interface.exceptions import BrokenImplementation
from thug.Plugins.IPlugin import IPlugin

log = logging.getLogger("Thug")

PLUGINS_PATH          = "/etc/thug/plugins"
HANDLER_NAME          = "Handler"
HANDLER_MODULE        = "%s.py" % (HANDLER_NAME, )
FIRST_LOW_PRIO        = 1000
PRE_ANALYSIS_PLUGINS  = 'PRE'
POST_ANALYSIS_PLUGINS = 'POST'

sys.path.append(PLUGINS_PATH)


class ThugPlugins(object):
    def __init__(self, phase, thug):
        self.phase            = phase
        self.thug             = thug
        self.plugins          = dict()
        self.last_low_prio    = FIRST_LOW_PRIO
        self.get_plugins()

    def __call__(self):
        self.run()

    def handle_low_prio_plugin(self):
        plugin_prio = self.last_low_prio
        self.last_low_prio += 1
        return plugin_prio

    def get_plugin_prio(self, plugin_info):
        if len(plugin_info) < 3:
            return self.handle_low_prio_plugin()

        try:
            plugin_prio = int(plugin_info[2])
        except ValueError:
            plugin_prio = self.handle_low_prio_plugin()

        return plugin_prio

    def get_plugins(self):
        plugins = dict()

        for p in os.listdir(PLUGINS_PATH):
            if not p.startswith(self.phase):
                continue

            pkg = os.path.join(PLUGINS_PATH, p)
            if not os.path.isdir(pkg):
                continue

            if HANDLER_MODULE not in os.listdir(pkg):
                continue

            plugin_info = p.split('-')
            if len(plugin_info) < 2:
                continue

            plugin_name = p
            plugin_prio = self.get_plugin_prio(plugin_info)

            plugins[plugin_name] = plugin_prio

        self.plugins = sorted(plugins.items(), key = lambda x: x[1])

    def run(self):
        for plugin in self.plugins:
            name, prio = plugin
            source = "%s.%s" % (name, HANDLER_NAME)

            module = __import__(source)
            components = source.split('.')[1:]
            for component in components:
                module = getattr(module, component)

            handler = getattr(module, "Handler", None)
            if handler:
                log.warning("[PLUGIN][%s] Phase: %s_ANALYSIS Priority: %d", name.split('-')[1], self.phase, prio)
                p = handler()
                try:
                    verifyObject(IPlugin, p)
                    p.run(self.thug, log)
                except BrokenImplementation as e:
                    log.warning("[%s] %s", source, e)




import logging

log = logging.getLogger("Thug")


def GetVersions(self):
    versions = ""
    for feature in ('Accessibility',
                    'AcroForm',
                    'Annots',
                    'Checkers',
                    'DigSig',
                    'DVA',
                    'eBook',
                    'EScript',
                    'HLS',
                    'IA32',
                    'MakeAccessible',
                    'Multimedia',
                    'PDDom',
                    'PPKLite',
                    'ReadOutLoud',
                    'reflow',
                    'SaveAsRTF',
                    'Search',
                    'Search5',
                    'SendMail',
                    'Spelling',
                    'Updater',
                    'weblink'):
        versions += "%s=%s," % (feature, log.ThugVulnModules.acropdf_pdf, )

    return versions



#!/usr/bin/env python
#
# Encoding.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA


import cchardet


class Encoding(object):
    def __init__(self):
        pass

    def detect(self, data, safe = False):
        try:
            return cchardet.detect(data)
        except:
            if safe:
                return None

            raise



#!/usr/bin/env python
#
# thug.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import sys
import os
import getopt
import logging

from .ThugAPI import ThugAPI
from .Plugins.ThugPlugins import ThugPlugins
from .Plugins.ThugPlugins import PRE_ANALYSIS_PLUGINS
from .Plugins.ThugPlugins import POST_ANALYSIS_PLUGINS

log = logging.getLogger("Thug")
log.setLevel(logging.WARN)


class Thug(ThugAPI):
    def __init__(self, args):
        self.args = args
        ThugAPI.__init__(self)

    def usage(self):
        msg = """
Synopsis:
    Thug: Pure Python honeyclient implementation

    Usage:
        thug [ options ] url

    Options:
        -h, --help              \tDisplay this help information
        -V, --version           \tDisplay Thug version
        -i, --list-ua           \tDisplay available user agents
        -u, --useragent=        \tSelect a user agent (use option -b for values, default: winxpie60)
        -e, --events=           \tEnable comma-separated specified DOM events handling
        -w, --delay=            \tSet a maximum setTimeout/setInterval delay value (in milliseconds)
        -n, --logdir=           \tSet the log output directory
        -o, --output=           \tLog to a specified file
        -r, --referer           \tSpecify a referer
        -p, --proxy=            \tSpecify a proxy (see below for format and supported schemes)
        -m, --attachment        \tSet the attachment mode
        -l, --local             \tAnalyze a locally saved page
        -x, --local-nofetch     \tAnalyze a locally saved page and prevent remote content fetching
        -v, --verbose           \tEnable verbose mode
        -d, --debug             \tEnable debug mode
        -q, --quiet             \tDisable console logging
        -a, --ast-debug         \tEnable AST debug mode (requires debug mode)
        -g, --http-debug        \tEnable HTTP debug mode
        -t, --threshold         \tMaximum pages to fetch
        -j, --extensive         \tExtensive fetch of linked pages
        -O, --connect-timeout   \tSet the connect timeout (in seconds, default: 10 seconds)
        -T, --timeout=          \tSet the analysis timeout (in seconds, default: 600 seconds)
        -c, --broken-url        \tSet the broken URL mode
        -y, --vtquery           \tQuery VirusTotal for samples analysis
        -s, --vtsubmit          \tSubmit samples to VirusTotal
        -b, --vt-apikey=        \tVirusTotal API key to be used at runtime
        -z, --web-tracking      \tEnable web client tracking inspection
        -k, --no-honeyagent     \tDisable HoneyAgent support

        Plugins:
        -A, --adobepdf=         \tSpecify the Adobe Acrobat Reader version (default: 9.1.0)
        -P, --no-adobepdf       \tDisable Adobe Acrobat Reader plugin
        -S, --shockwave=        \tSpecify the Shockwave Flash version (default: 10.0.64.0)
        -R, --no-shockwave      \tDisable Shockwave Flash plugin
        -J, --javaplugin=       \tSpecify the JavaPlugin version (default: 1.6.0.32)
        -K, --no-javaplugin     \tDisable Java plugin

        Classifiers:
        --htmlclassifier=       \tSpecify a list of additional (comma separated) HTML classifier rule files
        --urlclassifier=        \tSpecify a list of additional (comma separated) URL classifier rule files
        --jsclassifier=         \tSpecify a list of additional (comma separated) JS classifier rule files
        --vbsclassifier=        \tSpecify a list of additional (comma separated) VBS classifier rule files
        --sampleclassifier=     \tSpecify a list of additional (comma separated) Sample classifier rule files
        --textclassifier=       \tSpecify a list of additional (comma separated) Text classifier rule files
        --htmlfilter=           \tSpecify a list of additional (comma separated) HTML filter files
        --urlfilter=            \tSpecify a list of additional (comma separated) URL filter files
        --jsfilter=             \tSpecify a list of additional (comma separated) JS filter files
        --vbsfilter=            \tSpecify a list of additional (comma separated) VBS filter files
        --samplefilter=         \tSpecify a list of additional (comma separated) Sample filter files
        --textfilter=           \tSpecify a list of additional (comma separated) Text filter files

        Logging:
        -F, --file-logging      \tEnable file logging mode (default: disabled)
        -Z, --json-logging      \tEnable JSON logging mode (default: disabled)
        -M, --maec11-logging    \tEnable MAEC11 logging mode (default: disabled)
        -G, --elasticsearch-logging\tEnable ElasticSearch logging mode (default: disabled)
        -D, --mongodb-address=  \tSpecify address and port of the MongoDB instance (format: host:port)
        -Y, --no-code-logging   \tDisable code logging
        -U, --no-cert-logging   \tDisable SSL/TLS certificate logging

    Proxy Format:
        scheme://[username:password@]host:port (supported schemes: http, socks4, socks5)
"""

        print(msg)
        sys.exit(0)

    def list_ua(self):
        msg = """
Synopsis:
    Thug: Pure Python honeyclient implementation

    Available User-Agents:
"""

        for key, value in sorted(iter(log.ThugOpts.Personality.items()), key = lambda k_v: (k_v[1]['id'], k_v[0])):
            msg += "\t%s\t\t\t%s\n" % (key, value['description'], )

        print(msg)
        sys.exit(0)

    def analyze(self):
        p = getattr(self, 'run_remote', None)

        try:
            options, args = getopt.getopt(self.args,
                                          'hViu:e:w:n:o:r:p:myszklxvdqagA:PS:RJ:Kt:jO:T:cFZMGYUD:b:',
                ['help',
                'version',
                'list-ua',
                'useragent=',
                'events=',
                'delay=',
                'logdir=',
                'output=',
                'referer=',
                'proxy=',
                'attachment',
                'vtquery',
                'vtsubmit',
                'web-tracking',
                'no-honeyagent',
                'local',
                'local-nofetch',
                'verbose',
                'debug',
                'quiet',
                'ast-debug',
                'http-debug',
                'adobepdf=',
                'no-adobepdf',
                'shockwave=',
                'no-shockwave',
                'javaplugin=',
                'no-javaplugin',
                'threshold=',
                'extensive',
                'connect-timeout=',
                'timeout=',
                'broken-url',
                'htmlclassifier=',
                'urlclassifier=',
                'jsclassifier=',
                'vbsclassifier=',
                'sampleclassifier=',
                'textclassifier=',
                'htmlfilter=',
                'urlfilter=',
                'jsfilter=',
                'vbsfilter=',
                'samplefilter=',
                'textfilter='
                'file-logging',
                'json-logging',
                'maec11-logging',
                'elasticsearch-logging',
                'no-code-logging',
                'no-cert-logging',
                'mongodb-address=',
                'vt-apikey=',
                ])
        except getopt.GetoptError:
            self.usage()

        if not options and not args:
            self.usage()

        for option in options:
            if option[0] in ('-h', '--help'):
                self.usage()
            elif option[0] in ('-V', '--version'):
                self.version()
            elif option[0] in ('-i', '--list-ua'):
                self.list_ua()

        self.set_raise_for_proxy(False)

        for option in options:
            if option[0] in ('-u', '--useragent', ):
                self.set_useragent(option[1])
            elif option[0] in ('-e', '--events'):
                self.set_events(option[1])
            elif option[0] in ('-w', '--delay'):
                self.set_delay(option[1])
            elif option[0] in ('-r', '--referer', ):
                self.set_referer(option[1])
            elif option[0] in ('-p', '--proxy', ):
                self.set_proxy(option[1])
            elif option[0] in ('-m', '--attachment', ):
                self.set_attachment()
            elif option[0] in ('-y', '--vtquery', ):
                self.set_vt_query()
            elif option[0] in ('-s', '--vtsubmit', ):
                self.set_vt_submit()
            elif option[0] in ('-b', '--vt-apikey', ):
                self.set_vt_runtime_apikey(option[1])
            elif option[0] in ('-z', '--web-tracking', ):
                self.set_web_tracking()
            elif option[0] in ('-k', '--no-honeyagent', ):
                self.disable_honeyagent()
            elif option[0] in ('-l', '--local', ):
                p = getattr(self, 'run_local')
            elif option[0] in ('-x', '--local-nofetch', ):
                p = getattr(self, 'run_local')
                self.set_no_fetch()
            elif option[0] in ('-v', '--verbose', ):
                self.set_verbose()
            elif option[0] in ('-d', '--debug', ):
                self.set_debug()
            elif option[0] in ('-a', '--ast-debug', ):
                self.set_ast_debug()
            elif option[0] in ('-g', '--http-debug', ):
                self.set_http_debug()
            elif option[0] in ('-A', '--adobepdf', ):
                self.set_acropdf_pdf(option[1])
            elif option[0] in ('-P', '--no-adobepdf', ):
                self.disable_acropdf()
            elif option[0] in ('-S', '--shockwave', ):
                self.set_shockwave_flash(option[1])
            elif option[0] in ('-R', '--no-shockwave', ):
                self.disable_shockwave_flash()
            elif option[0] in ('-J', '--javaplugin', ):
                self.set_javaplugin(option[1])
            elif option[0] in ('-K', '--no-javaplugin', ):
                self.disable_javaplugin()
            elif option[0] in ('-t', '--threshold', ):
                self.set_threshold(option[1])
            elif option[0] in ('-j', '--extensive', ):
                self.set_extensive()
            elif option[0] in ('-O', '--connect-timeout', ):
                self.set_connect_timeout(option[1])
            elif option[0] in ('-T', '--timeout', ):
                self.set_timeout(option[1])
            elif option[0] in ('--htmlclassifier', ):
                for classifier in option[1].split(','):
                    self.add_htmlclassifier(os.path.abspath(classifier))
            elif option[0] in ('--urlclassifier', ):
                for classifier in option[1].split(','):
                    self.add_urlclassifier(os.path.abspath(classifier))
            elif option[0] in ('--jsclassifier', ):
                for classifier in option[1].split(','):
                    self.add_jsclassifier(os.path.abspath(classifier))
            elif option[0] in ('--vbsclassifier', ):
                for classifier in option[1].split(','):
                    self.add_vbsclassifier(os.path.abspath(classifier))
            elif option[0] in ('--sampleclassifier', ):
                for classifier in option[1].split(','):
                    self.add_sampleclassifier(os.path.abspath(classifier))
            elif option[0] in ('--textclassifier', ):
                for classifier in option[1].split(','):
                    self.add_textclassifier(os.path.abspath(classifier))
            elif option[0] in ('--htmlfilter', ):
                for f in option[1].split(','):
                    self.add_htmlfilter(os.path.abspath(f))
            elif option[0] in ('--urlfilter', ):
                for f in option[1].split(','):
                    self.add_urlfilter(os.path.abspath(f))
            elif option[0] in ('--jsfilter', ):
                for f in option[1].split(','):
                    self.add_jsfilter(os.path.abspath(f))
            elif option[0] in ('--vbsfilter', ):
                for f in option[1].split(','):
                    self.add_vbsfilter(os.path.abspath(f))
            elif option[0] in ('--samplefilter', ):
                for f in option[1].split(','):
                    self.add_samplefilter(os.path.abspath(f))
            elif option[0] in ('--textfilter', ):
                for f in option[1].split(','):
                    self.add_textfilter(os.path.abspath(f))
            elif option[0] in ('-c', '--broken-url', ):
                self.set_broken_url()
            elif option[0] in ('-F', '--file-logging', ):
                self.set_file_logging()
            elif option[0] in ('-Z', '--json-logging', ):
                self.set_json_logging()
            elif option[0] in ('-M', '--maec11-logging', ):
                self.set_maec11_logging()
            elif option[0] in ('-G', '--elasticsearch-logging', ):
                self.set_elasticsearch_logging()
            elif option[0] in ('-Y', '--no-code-logging', ):
                self.disable_code_logging()
            elif option[0] in ('-U', '--no-cert-logging', ):
                self.disable_cert_logging()
            elif option[0] in ('-D', '--mongodb-address', ):
                self.set_mongodb_address(option[1])

        self.log_init(args[0])

        for option in options:
            if option[0] in ('-n', '--logdir'):
                self.set_log_dir(option[1])
            elif option[0] in ('-o', '--output', ):
                self.set_log_output(option[1])
            elif option[0] in ('-q', '--quiet', ):
                self.set_log_quiet()

        if p:
            ThugPlugins(PRE_ANALYSIS_PLUGINS, self)()
            p(args[0])
            ThugPlugins(POST_ANALYSIS_PLUGINS, self)()

        self.log_event()
        return log


def main():
    if not os.getenv('THUG_PROFILE', None):
        Thug(sys.argv[1:])()
    else:
        try:
            from StringIO import StringIO
        except ImportError:
            from io import StringIO

        import cProfile
        import pstats

        profiler = cProfile.Profile()
        profiler.enable()
        Thug(sys.argv[1:])()
        profiler.disable()

        s  = StringIO()
        ps = pstats.Stats(profiler, stream = s).sort_stats('cumulative')
        ps.print_stats()
        with open('/tmp/thug-profiler.log', 'w') as fd:
            fd.write(s.getvalue())


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long


class HTMLAppletElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    align           = attr_property("align")
    alt             = attr_property("alt")
    archive         = attr_property("archive")
    code            = attr_property("code")
    codeBase        = attr_property("codebase")
    height          = attr_property("height")
    hspace          = attr_property("hspace", thug_long)
    name            = attr_property("name")
    object          = attr_property("object")
    vspace          = attr_property("vspace", thug_long)
    width           = attr_property("width")



# UUSee UUUpgrade ActiveX Control 'Update' Method Arbitrary File Download Vulnerability
# CVE...

import logging

log = logging.getLogger("Thug")


def Update(self, *args):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "UUsee UUPgrade ActiveX",
                                      "Attack in Update Method")



# NamoInstaller ActiveX Control 1.x - 3.x
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def Install(self, arg):
    if len(arg) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "NamoInstaller ActiveX",
                                          "Overflow in Install method")

    if str([arg]).find('http') > -1:
        log.ThugLogging.add_behavior_warn('[NamoInstaller ActiveX] Insecure download from URL %s' % (arg, ))
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "NamoInstaller ActiveX",
                                          "Insecure download from URL",
                                          forward = False,
                                          data = {
                                                    "url": arg
                                                 }
                                         )
        try:
            self._window._navigator.fetch(arg, redirect_type = "NamoInstaller Exploit")
        except:  # pylint:disable=bare-except
            log.ThugLogging.add_behavior_warn('[NamoInstaller ActiveX] Fetch failed')



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long


class HTMLBaseFontElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    color           = attr_property("color")
    face            = attr_property("face")
    size            = attr_property("size", thug_long)



#!/usr/bin/env python

# import bs4 as BeautifulSoup

import logging
log = logging.getLogger("Thug")


def text_property(readonly = False):
    def getter(self):
        return str(self.tag.string) if self.tag.string else ""

    def setter(self, text):
        # if self.tag.string:
        #    self.tag.contents[0] = BeautifulSoup.NavigableString(text)
        # else:
        #    self.tag.append(text)
        #
        # self.tag.string = self.tag.contents[0]
        self.tag.string = text
        if self.tagName.lower() in ('script', ):
            if log.ThugOpts.code_logging:
                log.ThugLogging.add_code_snippet(text, 'Javascript', 'Contained_Inside')

            self.doc.window.evalScript(text, self.tag.string)

    return property(getter) if readonly else property(getter, setter)



#!/usr/bin/env python
#
# MongoDB.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import os
import datetime
import logging
from .compatibility import thug_unicode

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser

MONGO_MODULE = True

try:
    import pymongo
    import gridfs
    from pymongo.errors import DuplicateKeyError
except ImportError:
    MONGO_MODULE = False

from .ExploitGraph import ExploitGraph

log = logging.getLogger("Thug")


class MongoDB(object):
    def __init__(self, thug_version):
        self.thug_version = thug_version
        self.enabled      = True

        if not self.__check_mongo_module():
            return

        if not self.__init_config():
            return

        self.__init_db()
        self.chain_id = self.make_counter(0)

    def __check_mongo_module(self):
        if not MONGO_MODULE:
            log.info('[MongoDB] MongoDB instance not available')
            self.enabled = False

        return self.enabled

    def __init_config(self):
        self.opts = dict()

        if log.ThugOpts.mongodb_address:
            try:
                self.opts['host'] = log.ThugOpts.mongodb_address
                self.opts['enable'] = 'True'
                return True
            except:  # pylint:disable=bare-except
                log.warning("Invalid MongoDB address specified at runtime, using default values instead (if any)")

        config = ConfigParser.ConfigParser()

        conf_file = os.path.join(log.configuration_path, 'logging.conf')

        if not os.path.exists(conf_file):
            conf_file = os.path.join(log.configuration_path, 'logging.conf.default')

        if not os.path.exists(conf_file):
            self.enabled = False
            return False

        config.read(conf_file)

        for option in config.options('mongodb'):
            self.opts[option] = config.get('mongodb', option)

        if self.opts['enable'].lower() in ('false', ):
            self.enabled = False

        return self.enabled

    def __init_db(self):
        # MongoDB Connection class is marked as deprecated (MongoDB >= 2.4).
        # The following code tries to use the new MongoClient if available and
        # reverts to the old Connection class if not. This code will hopefully
        # disappear in the next future.
        client = getattr(pymongo, 'MongoClient', None)
        if client is None:
            client = getattr(pymongo, 'Connection', None)

        try:
            connection = client(self.opts['host'])
        except:  # pylint:disable=bare-except
            log.warning('[MongoDB] MongoDB instance not available')
            self.enabled = False
            return

        db                = connection.thug
        self.urls         = db.urls
        self.analyses     = db.analyses
        self.locations    = db.locations
        self.connections  = db.connections
        self.graphs       = db.graphs
        self.samples      = db.samples
        self.behaviors    = db.behaviors
        self.certificates = db.certificates
        self.virustotal   = db.virustotal
        self.honeyagent   = db.honeyagent
        self.androguard   = db.androguard
        self.peepdf       = db.peepdf
        self.exploits     = db.exploits
        self.classifiers  = db.classifiers
        self.codes        = db.codes
        self.maec11       = db.maec11
        self.json         = db.json
        dbfs              = connection.thugfs
        self.fs           = gridfs.GridFS(dbfs)

        self.__build_indexes()

    def __build_indexes(self):
        self.urls.ensure_index('url', unique = True)

    def make_counter(self, p):
        _id = p
        while True:
            yield _id
            _id += 1

    def __get_url(self, url):
        entry = self.urls.find_one({'url': url})
        if entry:
            return entry['_id']

        return None

    def get_url(self, url):
        entry = self.__get_url(url)
        if entry:
            return entry

        try:
            entry = self.urls.insert({'url' : url})
        except DuplicateKeyError:
            entry = self.__get_url(url)

        return entry

    def set_url(self, url):
        if not self.enabled:
            return

        self.graph  = ExploitGraph(url)

        self.url_id = self.get_url(url)
        if self.url_id is None:
            log.warning('[MongoDB] MongoDB internal error')
            self.enabled = False
            return

        analysis = {
            "url_id"      : self.url_id,
            "timestamp"   : str(datetime.datetime.now()),
            "thug"        : {
                                "version"            : self.thug_version,
                                "personality" : {
                                    "useragent"      : log.ThugOpts.useragent
                                },
                                "plugins" : {
                                    "acropdf"        : self.get_vuln_module("acropdf"),
                                    "javaplugin"     : self.get_vuln_module("_javaplugin"),
                                    "shockwaveflash" : self.get_vuln_module("shockwave_flash")
                                },
                                "options" : {
                                    "local"          : log.ThugOpts.local,
                                    "nofetch"        : log.ThugOpts.no_fetch,
                                    "proxy"          : log.ThugOpts._proxy,
                                    "events"         : log.ThugOpts.events,
                                    "delay"          : log.ThugOpts.delay,
                                    "referer"        : log.ThugOpts.referer,
                                    "timeout"        : log.ThugOpts.timeout,
                                    "threshold"      : log.ThugOpts.threshold,
                                    "extensive"      : log.ThugOpts.extensive,
                                },
                            }
        }

        self.analysis_id = self.analyses.insert(analysis)
        log.warning('[MongoDB] Analysis ID: %s', str(self.analysis_id))

    def get_vuln_module(self, module):
        disabled = getattr(log.ThugVulnModules, "%s_disabled" % (module, ), True)
        if disabled:
            return "disabled"

        return getattr(log.ThugVulnModules, module)

    def log_location(self, url, data, flags = None):
        if not self.enabled:
            return

        if flags is None:
            flags = dict()

        content    = data.get("content", None)
        content_id = self.fs.put(content,
            mtype  = data.get("mtype", None)
        ) if content else None

        location = {
            'analysis_id'   : self.analysis_id,
            'url_id'        : self.get_url(url),
            'status'        : data.get("status", None),
            "content_id"    : content_id,
            'content-type'  : data.get("ctype", None),
            'md5'           : data.get("md5", None),
            'sha256'        : data.get("sha256", None),
            'flags'         : flags,
            'size'          : data.get("fsize", None),
            'mime-type'     : data.get("mtype", None)
        }

        self.locations.insert(location)

    def log_connection(self, source, destination, method, flags = None):
        if not self.enabled:
            return

        if flags is None:
            flags = dict()

        connection = {
            'analysis_id'    : self.analysis_id,
            'chain_id'       : next(self.chain_id),
            'source_id'      : self.get_url(source),
            'destination_id' : self.get_url(destination),
            'method'         : method,
            'flags'          : flags
        }

        self.connections.insert(connection)
        self.graph.add_connection(source, destination, method)

    def log_exploit_event(self, url, module, description, cve = None, data = None):
        """
        Log file information for a given url

        @url            URL where this exploit occured
        @module         Module/ActiveX Control, ... that gets exploited
        @description    Description of the exploit
        @cve            CVE number (if available)
        """
        if not self.enabled:
            return

        exploit = {
            'analysis_id' : self.analysis_id,
            'url_id'      : self.get_url(url),
            'module'      : module,
            'description' : description,
            'cve'         : cve,
            'data'        : data
        }

        self.exploits.insert(exploit)

    def log_classifier(self, classifier, url, rule, tags):
        """
        Log classifiers matching for a given url

        @classifier     Classifier name
        @url            URL where the rule match occurred
        @rule           Rule name
        @tags           Rule tags
        """
        if not self.enabled:
            return

        classification = {
            'analysis_id' : self.analysis_id,
            'url_id'      : self.get_url(url),
            'classifier'  : classifier,
            'rule'        : rule,
            'tags'        : tags
        }

        self.classifiers.insert(classification)

    def get_url_from_location(self, md5):
        result = self.locations.find_one({'analysis_id' : self.analysis_id,
                                          'md5'         : md5})
        if not result:
            return None

        return result['url_id']

    def log_file(self, data, url = None, params = None):
        if not self.enabled:
            return

        r = dict(data)

        r['sample_id'] = self.fs.put(data['data'])
        r.pop('data', None)

        if url:
            url_id = self.get_url(url)
            r.pop('url', None)
        else:
            url_id = self.get_url_from_location(data['md5'])

        r['analysis_id'] = self.analysis_id
        r['url_id']      = url_id

        self.samples.insert(r)

    def log_maec11(self, basedir):
        if not self.enabled:
            return

        if not log.ThugOpts.maec11_logging:
            return

        p = log.ThugLogging.modules.get('maec11', None)
        if p is None:
            return

        m = getattr(p, 'get_maec11_data', None)
        if m is None:
            return

        report = m(basedir)
        analysis = {
            'analysis_id'   : self.analysis_id,
            'report'        : report
        }

        self.maec11.insert(analysis)

    def log_json(self, basedir):
        if not self.enabled:
            return

        if not log.ThugOpts.json_logging:
            return

        p = log.ThugLogging.modules.get('json', None)
        if p is None:
            return

        m = getattr(p, 'get_json_data', None)
        if m is None:
            return

        report = m(basedir)
        analysis = {
            'analysis_id'   : self.analysis_id,
            'report'        : report
        }

        self.json.insert(analysis)

    def log_event(self, basedir):
        if not self.enabled:
            return

        self.log_maec11(basedir)
        self.log_json(basedir)

        G = self.graph.draw()
        if G is None:
            return

        graph = {
            'analysis_id'   : self.analysis_id,
            'graph'         : G
        }

        self.graphs.insert(graph)

    def fix(self, data):
        """
        Fix encoding of data

        @data  data to encode properly
        """
        try:
            enc = log.Encoding.detect(data)
            return data.decode(enc['encoding']).replace("\n", "").strip()
        except:  # pylint:disable=bare-except
            return thug_unicode(data).replace("\n", "").strip()

    def add_code_snippet(self, snippet, language, relationship, method = "Dynamic Analysis"):
        if not self.enabled:
            return

        code = {
            'analysis_id'  : self.analysis_id,
            'snippet'      : self.fix(snippet),
            'language'     : self.fix(language),
            'relationship' : self.fix(relationship),
            'method'       : self.fix(method)
        }

        self.codes.insert(code)

    def add_behavior(self, description = None, cve = None, method = "Dynamic Analysis"):
        if not self.enabled:
            return

        if not cve and not description:
            return

        behavior = {
            'analysis_id' : self.analysis_id,
            'description' : self.fix(description),
            'cve'         : self.fix(cve),
            'method'      : self.fix(method),
            'timestamp'   : str(datetime.datetime.now())
        }

        self.behaviors.insert(behavior)

    def add_behavior_warn(self, description = None, cve = None, method = "Dynamic Analysis"):
        if not self.enabled:
            return

        self.add_behavior(description, cve, method)

    def log_certificate(self, url, certificate):
        if not self.enabled:
            return

        certificate = {
            'analysis_id' : self.analysis_id,
            'url_id'      : self.get_url(url),
            'certificate' : certificate
        }

        self.certificates.insert(certificate)

    def log_analysis_module(self, collection, sample, report):
        if not self.enabled:
            return

        s = self.samples.find_one({'analysis_id' : self.analysis_id,
                                   'md5'         : sample['md5'],
                                   'sha1'        : sample['sha1']})
        if not s:
            return

        r = {
            'analysis_id' : self.analysis_id,
            'sample_id'   : s['_id'],
            'report'      : report
        }

        collection.insert(r)

    def log_virustotal(self, sample, report):
        self.log_analysis_module(self.virustotal, sample, report)

    def log_honeyagent(self, sample, report):
        self.log_analysis_module(self.honeyagent, sample, report)

    def log_androguard(self, sample, report):
        self.log_analysis_module(self.androguard, sample, report)

    def log_peepdf(self, sample, report):
        self.log_analysis_module(self.peepdf, sample, report)



#!/usr/bin/env python
#
# VBSClassifier.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
from .BaseClassifier import BaseClassifier

log = logging.getLogger("Thug")


class VBSClassifier(BaseClassifier):
    default_rule_file   = "rules/vbsclassifier.yar"
    default_filter_file = "rules/vbsfilter.yar"
    classifier          = "VBS Classifier"

    def __init__(self):
        BaseClassifier.__init__(self)

    def classify(self, url, script):
        for match in self.rules.match(data = script):
            self.matches.append((url, match))

            rule = match.rule
            tags = ",".join([" ".join(t.split('_')) for t in match.tags])
            log.ThugLogging.log_classifier("vbs", url, rule, tags)

    def filter(self, url, script):
        ret = False

        for match in self.filters.match(data = script):
            rule = match.rule
            tags = ", ".join([" ".join(t.split('_')) for t in match.tags])
            log.ThugLogging.log_classifier("vbsfilter", url, rule, tags)
            ret = True

        return ret



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLBaseElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    href            = attr_property("href")
    target          = attr_property("target")



#!/usr/bin/env python


def attr_property(name, attrtype = str, readonly = False, default = None):
    def getter(self):
        return attrtype(self.tag[name]) if self.tag.has_attr(name) else default

    def setter(self, value):
        self.tag[name] = attrtype(value)

    return property(getter) if readonly else property(getter, setter)



#!/usr/bin/env python
#
# Storage.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
from collections import OrderedDict
from .JSClass import JSClass
from .W3C.Events.StorageEvent import StorageEvent

log = logging.getLogger("Thug")


class Storage(OrderedDict, JSClass):
    def __init__(self, *args, **kwargs):
        super(Storage, self).__init__(*args, **kwargs)

    def __str__(self):
        return "[object Storage]"

    def __unicode__(self):
        return "[object Storage]"

    @property
    def length(self):
        return len(self)

    def key(self, index):
        if index > self.length:
            return None

        return self.keys()[index - 1]

    def getItem(self, key):
        try:
            return super(Storage, self).__getitem__(key)
        except KeyError:
            return None

    def __setitem__(self, key, value, dict_setitem = dict.__setitem__):
        self.setItem(key, value)

    def setItem(self, key, value):
        oldvalue = self[key] if key in self else None
        super(Storage, self).__setitem__(key, value)
        log.WebTracking.inspect_storage_setitem(self, key, value)

        evtObject = StorageEvent('storage', log.DFT.window)
        evtObject.initStorageEvent('storage',
                                   False,
                                   False,
                                   key,
                                   oldvalue,
                                   value,
                                   log.DFT.window.url,
                                   self)

        log.DFT.handle_window_storage_event('onstorage', evtObject)

    def __delitem__(self, key, dict_delitem = dict.__delitem__):
        self.removeItem(key)

    def removeItem(self, key):
        oldvalue = self[key] if key in self else None
        super(Storage, self).__delitem__(key)
        log.WebTracking.inspect_storage_removeitem(self, key)

        evtObject = StorageEvent('storage', log.DFT.window)
        evtObject.initStorageEvent('storage',
                                   False,
                                   False,
                                   key,
                                   oldvalue,
                                   None,
                                   log.DFT.window.url,
                                   self)

        log.DFT.handle_window_storage_event('onstorage', evtObject)

    def clear(self):
        super(Storage, self).clear()
        self.__init__()
        log.WebTracking.inspect_storage_clear(self)

        evtObject = StorageEvent('storage', log.DFT.window)
        evtObject.initStorageEvent('storage',
                                   False,
                                   False,
                                   None,
                                   None,
                                   None,
                                   log.DFT.window.url,
                                   self)

        log.DFT.handle_window_storage_event('onstorage', evtObject)



# Gogago YouTube Video Converter Buffer Overflow
# HTB23012

import logging

log = logging.getLogger("Thug")


def Download(self, arg):
    if len(arg) > 1024:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "Gogago YouTube Video Converter ActiveX",
                                          "Buffer Overflow")



#!/usr/bin/env python
#
# Console.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging

log = logging.getLogger("Thug")

from .JSClass import JSClass


class Console(JSClass):
    def __init__(self):
        self.__init_personality()

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        pass

    def __init_personality_Firefox(self):
        pass

    def __init_personality_Chrome(self):
        pass

    def __init_personality_Safari(self):
        pass

    def __init_personality_Opera(self):
        pass

    def debug(self, *args):
        pass

    # def dir(self, *args):
    #    pass

    # def error(self, *args):
    #    pass

    def group(self):
        pass

    def groupCollapsed(self):
        pass

    def groupEnd(self):
        pass

    # def info(self, *args):
    #    pass

    # def log(self, *args):
    #    pass

    def time(self, timerName):
        pass

    def timeEnd(self, timerName):
        pass

    def trace(self):
        pass

    def warn(self, *args):
        pass



#!/usr/bin/env python
#
# Encoding.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import magic


class Magic(object):
    def __init__(self, data):
        self.data = data

    def get_mime(self):
        try:
            # This works with python-magic >= 0.4.6 from pypi
            mtype = magic.from_buffer(self.data, mime = True)
        except:  # pylint:disable=bare-except
            try:
                # Ubuntu workaround
                # This works with python-magic >= 5.22 from Ubuntu (apt)
                ms = magic.open(magic.MAGIC_MIME)
                ms.load()
                mtype = ms.buffer(self.data).split(';')[0]
            except:  # pylint:disable=bare-except
                # Filemagic workaround
                # This works with filemagic >= 1.6 from pypi
                with magic.Magic(flags = magic.MAGIC_MIME_TYPE) as m:  # pylint:disable=unexpected-keyword-arg
                    mtype = m.id_buffer(self.data)

        return mtype



# Microsoft DirectShow MPEG2TuneRequest Component Stack Overflow(MS09-032)
# CVE-2008-0015,CVE-2008-0020

import logging

log = logging.getLogger("Thug")


def Setdata(self, val):
    self.__dict__['data'] = val
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "Microsoft DirectShow MPEG2TuneRequest ActiveX",
                                      "Stack Overflow in data property",
                                      cve = 'CVE-2008-0015')

    log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2008-0015", None)



# ICQ Toolbar attack
# CVE-NOMATCH

import logging

log = logging.getLogger("Thug")


def GetPropertyById(self, arg0, arg1):
    if len(arg1) > 120:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "ICQ Toolbar ActiveX",
                                          "Buffer overflow in GetPropertyById")



__all__ = ['HTMLAnchorElement',
           'HTMLAppletElement',
           'HTMLBRElement',
           'HTMLBaseElement',
           'HTMLBaseFontElement',
           'HTMLBodyElement',
           'HTMLButtonElement',
           'HTMLCollection',
           'HTMLDListElement',
           'HTMLDirectoryElement',
           'HTMLDivElement',
           'HTMLDocument',
           'HTMLElement',
           'HTMLFieldSetElement',
           'HTMLFontElement',
           'HTMLFormElement',
           'HTMLFrameElement',
           'HTMLFrameSetElement',
           'HTMLHRElement',
           'HTMLHeadElement',
           'HTMLHeadingElement',
           'HTMLHtmlElement',
           'HTMLIFrameElement',
           'HTMLImageElement',
           'HTMLInputElement',
           'HTMLIsIndexElement',
           'HTMLLIElement',
           'HTMLLabelElement',
           'HTMLLegendElement',
           'HTMLLinkElement',
           'HTMLMenuElement',
           'HTMLMetaElement',
           'HTMLModElement',
           'HTMLOListElement',
           'HTMLObjectElement',
           'HTMLOptGroupElement',
           'HTMLOptionElement',
           'HTMLOptionsCollection',
           'HTMLParagraphElement',
           'HTMLParamElement',
           'HTMLPreElement',
           'HTMLQuoteElement',
           'HTMLScriptElement',
           'HTMLSelectElement',
           'HTMLStyleElement',
           'HTMLTableCaptionElement',
           'HTMLTableCellElement',
           'HTMLTableColElement',
           'HTMLTableElement',
           'HTMLTableRowElement',
           'HTMLTableSectionElement',
           'HTMLTextAreaElement',
           'HTMLTitleElement',
           'HTMLUListElement',
           'TAnimateColor']



#!/usr/bin/env python
#
# History.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging

log = logging.getLogger("Thug")

from .JSClass import JSClass
from .Alexa import Alexa


class History(JSClass):
    def __init__(self, window):
        self._window  = window
        self.urls     = Alexa
        self.pos      = None

        self.__init_personality()

    def __init_personality(self):
        self._navigationMode = "automatic"

        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        pass

    def __init_personality_Firefox(self):
        self.current  = self._current
        self.next     = self._next
        self.previous = self._previous

    def __init_personality_Chrome(self):
        pass

    def __init_personality_Safari(self):
        pass

    def __init_personality_Opera(self):
        self.current        = self._current
        self.navigationMode = property(self._get_navigationMode, self._set_navigationMode)

    @property
    def window(self):
        return self._window

    @property
    def length(self):
        return len(self.urls)

    @property
    def _current(self):
        if self.pos:
            return self.urls[self.pos]
        return None

    @property
    def _next(self):
        if self.pos and len(self.urls) > self.pos + 1:
            return self.urls[self.pos + 1]
        return None

    @property
    def _previous(self):
        if self.pos and self.pos > 0:
            return self.urls[self.pos - 1]
        return None

    def _get_navigationMode(self):
        return self._navigationMode

    def _set_navigationMode(self, value):
        if value in ("automatic", "compatible", "fast", ):
            self._navigationMode = value

    def back(self):
        """Loads the previous URL in the history list"""
        return self.go(-1)

    def forward(self):
        """Loads the next URL in the history list"""
        return self.go(1)

    def go(self, num_or_url):
        """Loads a specific URL from the history list"""
        try:
            off = int(num_or_url)

            self.pos += off
            self.pos = min(max(0, self.pos), len(self.urls) - 1)
            self._window.open(self.urls[self.pos])
        except ValueError:
            self._window.open(num_or_url)

    def update(self, url, replace = False):
        if self.pos is None:
            self.urls.append(url)
            self.pos = 0
        elif replace:
            self.urls[self.pos] = url
        elif self.urls[self.pos] != url:
            self.urls = self.urls[:self.pos + 1]
            self.urls.append(url)
            self.pos += 1



#!/usr/bin/env python

import logging
import bs4 as BeautifulSoup
from .HTMLElement import HTMLElement
from .HTMLCollection import HTMLCollection
from .HTMLTableSectionElement import HTMLTableSectionElement
from .HTMLTableCaptionElement import HTMLTableCaptionElement
from .attr_property import attr_property

log = logging.getLogger("Thug")


class HTMLTableElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)
        self._caption = None
        self._tHead   = None
        self._tFoot   = None
        self._rows    = HTMLCollection(doc, list())
        self._tBodies = HTMLCollection(doc, list())

    @property
    def caption(self):
        return self._caption

    @property
    def tHead(self):
        return self._tHead

    @property
    def tFoot(self):
        return self._tFoot

    @property
    def rows(self):
        return self._rows

    @property
    def tBodies(self):
        return self._tBodies

    align           = attr_property("align")
    bgColor         = attr_property("bgcolor")
    border          = attr_property("border")
    cellPadding     = attr_property("cellpadding")
    cellSpacing     = attr_property("cellspacing")
    frame           = attr_property("frame")
    rules           = attr_property("rules")
    summary         = attr_property("summary")
    width           = attr_property("width")

    def createTHead(self):
        if self._tHead:
            return self._tHead

        self._tHead = HTMLTableSectionElement(self.doc, BeautifulSoup.Tag(self.doc, name = 'thead'))
        self.rows.nodes.insert(0, self._tHead)
        return self._tHead

    def deleteTHead(self):
        if self.tHead:
            self._tHead = None
            del self.rows.nodes[0]

    def createTFoot(self):
        if self._tFoot:
            return self._tFoot

        self._tFoot = HTMLTableSectionElement(self.doc, BeautifulSoup.Tag(self.doc, name = 'tfoot'))
        self.rows.nodes.append(self._tFoot)
        return self._tFoot

    def deleteTFoot(self):
        if self._tFoot:
            self._tFoot = None
            del self.rows.nodes[-1]

    def createCaption(self):
        if self._caption:
            return self._caption

        self._caption = HTMLTableCaptionElement(self.doc, BeautifulSoup.Tag(self.doc, name = 'caption'))
        return self._caption

    def deleteCaption(self):
        if self.caption:
            self._caption = None

    # Modified in DOM Level 2
    def insertRow(self, index = None):
        # Insert a new empty row in the table. The new row is inserted immediately before
        # and in the same section as the current indexth row in the table. If index is -1
        # or equal to the number of rows, the new row is appended. In addition, when the
        # table is empty the row is inserted into a TBODY which is created and inserted
        # into the table.

        # `index' specifies the position of the row to insert (starts at 0). The value of
        # -1 can also be used; which result in that the new row will be inserted at the
        # last position. This parameter is required in Firefox and Opera, but optional in
        # Internet Explorer, Chrome and Safari. If this parameter is omitted, insertRow()
        # inserts a new row at the last position in IE and at the first position in Chrome
        # and Safari.
        if index is None:
            if log.ThugOpts.Personality.isIE():
                index = -1
            if log.ThugOpts.Personality.isChrome() or log.ThugOpts.Personality.isSafari():
                index = 0

        # PLEASE REVIEW ME!
        if not len(self.tBodies):
            tBody = HTMLTableSectionElement(self.doc, BeautifulSoup.Tag(self.doc, name = 'tbody'))
            self.tBodies.nodes.append(tBody)
            if self.tFoot is None:
                self.rows.nodes.append(tBody)
            else:
                self.rows.nodes.insert(-2, tBody)
        else:
            tBody = self.tBodies[-1]

        row = tBody.insertRow(index)
        return row



#!/usr/bin/env python

try:
    from io import StringIO
except ImportError:
    try:
        from cStringIO import StringIO
    except ImportError:
        from StringIO import StringIO

import bs4 as BeautifulSoup
import logging

from thug.DOM.W3C.DOMException import DOMException
from thug.DOM.W3C.Element import Element
from thug.DOM.W3C.Style.CSS.ElementCSSInlineStyle import ElementCSSInlineStyle
from .attr_property import attr_property

log = logging.getLogger("Thug")


class HTMLElement(Element, ElementCSSInlineStyle):
    id              = attr_property("id")
    title           = attr_property("title")
    lang            = attr_property("lang")
    dir             = attr_property("dir")
    className       = attr_property("class", default = "")

    def __init__(self, doc, tag):
        Element.__init__(self, doc, tag)
        ElementCSSInlineStyle.__init__(self, doc, tag)

    def __getattr__(self, key):
        if key in log.DFT.handled_on_events:
            return None

        if key in log.DFT._on_events:
            return None

        log.info("[HTMLElement] Undefined: %s", key)
        raise AttributeError

    def getInnerHTML(self):
        if not self.hasChildNodes():
            return ""

        html = StringIO()

        for tag in self.tag.contents:
            html.write(unicode(tag))

        return html.getvalue()

    def setInnerHTML(self, html):
        self.tag.clear()

        soup = BeautifulSoup.BeautifulSoup(html, "html5lib")

        for node in list(soup.head.descendants):
            self.tag.append(node)

            name = getattr(node, 'name', None)
            if name is None:
                continue

            handler = getattr(log.DFT, 'handle_%s' % (name, ), None)
            if handler:
                handler(node)

        for node in list(soup.body.children):
            self.tag.append(node)

            name = getattr(node, 'name', None)
            if name is None:
                continue

            handler = getattr(log.DFT, 'handle_%s' % (name, ), None)
            if handler:
                handler(node)

        # soup.head.unwrap()
        # soup.body.unwrap()
        # soup.html.wrap(self.tag)
        # self.tag.html.unwrap()

        for node in self.tag.descendants:
            name = getattr(node, 'name', None)
            if not name:
                continue

            p = getattr(self.doc.window.doc.DFT, 'handle_%s' % (name, ), None)
            if p is None:
                p = getattr(log.DFT, 'handle_%s' % (name, ), None)

            if p:
                p(node)

    innerHTML = property(getInnerHTML, setInnerHTML)

    # WARNING: NOT DEFINED IN W3C SPECS!
    def focus(self):
        pass

    @property
    def sourceIndex(self):
        return None

    def insertAdjacentHTML(self, position, text):
        if position not in ('beforebegin', 'afterbegin', 'beforeend', 'afterend', ):
            raise DOMException(DOMException.NOT_SUPPORTED_ERR)

        if position in ('beforebegin', ):
            target = self.tag.parent if self.tag.parent else self.doc.find('body')
            pos    = target.index(self.tag) - 1
        if position in ('afterbegin', ):
            target = self.tag
            pos    = 0
        if position in ('beforeend' ):
            target = self.tag
            pos    = len(list(self.tag.children))
        if position in ('afterend', ):
            target = self.tag.parent if self.tag.parent else self.doc.find('body')
            pos    = target.index(self.tag) + 1

        for node in BeautifulSoup.BeautifulSoup(text, "html.parser").contents:
            target.insert(pos, node)
            pos += 1



#!/usr/bin/env python
#
# Sidebar.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

from .JSClass import JSClass

import logging
log = logging.getLogger("Thug")


class Sidebar(JSClass):
    def __init__(self):
        self._providers  = set()
        self._engines    = set()
        self._favorites  = set()
        self._generators = set()

    def addMicrosummaryGenerator(self, generatorURL):
        self._generators.add(generatorURL)

    def addPanel(self, title, URL, customizeURL):
        self._favorites.add((title, URL, customizeURL))

    def addPersistentPanel(self, title, URL, customizeURL):
        self._favorites.add((title, URL, customizeURL))

    def addSearchEngine(self, engineURL, iconURL, message, suggestedCategory):
        self._enginess.add((engineURL, iconURL, message, suggestedCategory))

    def AddSearchProvider(self, URL):
        self._providers.add(URL)

    def IsSearchProviderInstalled(self, URL):
        if URL in self._providers:
            return 1  # A matching search provider is installed, but it is not the default.

        return 0    # No installed search provider was found with the specified prefix



#!/usr/bin/env python

from .HTMLElement import HTMLElement
from .attr_property import attr_property


class HTMLParamElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    name            = attr_property("name")
    type            = attr_property("type")
    value           = attr_property("value")
    valueType       = attr_property("valuetype")




import logging

log = logging.getLogger("Thug")


def CreateObject(self, _object, param = ''):
    import thug.ActiveX as ActiveX

    log.ThugLogging.add_behavior_warn("[VsmIDE.DTE ActiveX] CreateObject (%s)" % (_object))
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "VsmIDE.DTE ActiveX",
                                      "CreateObject",
                                      data = {
                                                "object": _object
                                             },
                                      forward = False)

    return ActiveX.ActiveX._ActiveXObject(self._window, _object)



#!/usr/bin/env python

import re
import string
import bs4 as BeautifulSoup

from .HTMLCollection import HTMLCollection


def xpath_property(xpath, readonly = False):
    RE_INDEXED = re.compile("(\w+)\[([^\]]+)\]")

    parts = xpath.split('/')

    def getChildren(tag, parts, recursive = False):
        if len(parts) == 0:
            return [tag]

        part = parts[0]

        if part == '':
            return getChildren(tag, parts[1:], True)

        if part == 'text()':
            return [tag.string]

        m = RE_INDEXED.match(part)

        if m:
            name = m.group(1)
            idx = m.group(2)
        else:
            name = part
            idx = None

        children = []

        _tags = tag.find_all(name, recursive = recursive)

        if idx:
            if idx[0] == '@':
                tags = [t for t in _tags if t.has_attr(idx[1:])]
            else:
                tags = [_tags[int(idx) - 1]]
        else:
            tags = _tags

        for child in tags:
            children += getChildren(child, parts[1:])

        return children

    def getter(self):
        children = getChildren(self.doc, parts)

        if xpath == '/html/body[1]' and not children:
            children = [self.doc]

        if parts[-1] == 'text()':
            return "".join(children)

        m = RE_INDEXED.match(parts[-1])

        if m:
            try:
                from thug.DOM.W3C.DOMImplementation import DOMImplementation
                string.atoi(m.group(2))

                return DOMImplementation.createHTMLElement(self.doc, children[0]) if len(children) > 0 else None
            except ValueError:
                pass

        return HTMLCollection(self.doc, children)

    def setter(self, value):
        tag = self.doc

        for part in parts:
            if part == '':
                continue
            elif part == 'text()':
                if tag.string:
                    tag.contents[0] = BeautifulSoup.NavigableString(value)
                else:
                    tag.append(value)
                    tag.string = tag.contents[0]
                return
            else:
                child = tag.find(part)

                if not child:
                    child = BeautifulSoup.Tag(parser = self.doc, name = part)

                    tag.append(child)

                tag = child

        tag.append(value)

    return property(getter) if readonly else property(getter, setter)



# WinZip FileView ActiveX Control
# CVE-2006-3890,CVE-2006-5198,CVE-2006-6884

import logging

log = logging.getLogger("Thug")


def CreateNewFolderFromName(self, arg):
    if len(arg) > 230:
        log.ThugLogging.log_exploit_event(self._window.url,
                                          "WinZip ActiveX",
                                          "CreateNewFolderFromName Overflow",
                                          cve = 'CVE-2006-6884')

        log.ThugLogging.log_classifier("exploit", log.ThugLogging.url, "CVE-2006-6884", None)




import logging

log = logging.getLogger("Thug")


def Setcachefolder(self, val):
    log.ThugLogging.log_exploit_event(self._window.url,
                                      "CreativeSoft ActiveX",
                                      "Overflow in cachefolder property")



#!/usr/bin/env python

import logging

log = logging.getLogger("Thug")


# Introduced in DOM Level 2
class AbstractView(object):
    def __init__(self):
        pass

    @property
    def document(self):
        return None



#!/usr/bin/env python

from thug.DOM.W3C.HTML import HTMLAnchorElement
from thug.DOM.W3C.HTML import HTMLAppletElement
from thug.DOM.W3C.HTML import HTMLBRElement
from thug.DOM.W3C.HTML import HTMLBaseElement
from thug.DOM.W3C.HTML import HTMLBaseFontElement
from thug.DOM.W3C.HTML import HTMLBodyElement
from thug.DOM.W3C.HTML import HTMLButtonElement
# from thug.DOM.W3C.HTML import HTMLCollection
from thug.DOM.W3C.HTML import HTMLDListElement
from thug.DOM.W3C.HTML import HTMLDirectoryElement
from thug.DOM.W3C.HTML import HTMLDivElement
from thug.DOM.W3C.HTML import HTMLDocument
from thug.DOM.W3C.HTML import HTMLElement
from thug.DOM.W3C.HTML import HTMLFieldSetElement
from thug.DOM.W3C.HTML import HTMLFontElement
from thug.DOM.W3C.HTML import HTMLFormElement
from thug.DOM.W3C.HTML import HTMLFrameElement
from thug.DOM.W3C.HTML import HTMLFrameSetElement
from thug.DOM.W3C.HTML import HTMLHRElement
from thug.DOM.W3C.HTML import HTMLHeadElement
from thug.DOM.W3C.HTML import HTMLHeadingElement
from thug.DOM.W3C.HTML import HTMLHtmlElement
from thug.DOM.W3C.HTML import HTMLIFrameElement
from thug.DOM.W3C.HTML import HTMLImageElement
from thug.DOM.W3C.HTML import HTMLInputElement
from thug.DOM.W3C.HTML import HTMLIsIndexElement
from thug.DOM.W3C.HTML import HTMLLIElement
from thug.DOM.W3C.HTML import HTMLLabelElement
from thug.DOM.W3C.HTML import HTMLLegendElement
from thug.DOM.W3C.HTML import HTMLLinkElement
from thug.DOM.W3C.HTML import HTMLMenuElement
from thug.DOM.W3C.HTML import HTMLMetaElement
from thug.DOM.W3C.HTML import HTMLModElement
from thug.DOM.W3C.HTML import HTMLOListElement
from thug.DOM.W3C.HTML import HTMLObjectElement
from thug.DOM.W3C.HTML import HTMLOptGroupElement
from thug.DOM.W3C.HTML import HTMLOptionElement
# from thug.DOM.W3C.HTML import HTMLOptionsCollection
from thug.DOM.W3C.HTML import HTMLParagraphElement
from thug.DOM.W3C.HTML import HTMLParamElement
from thug.DOM.W3C.HTML import HTMLPreElement
from thug.DOM.W3C.HTML import HTMLQuoteElement
from thug.DOM.W3C.HTML import HTMLScriptElement
from thug.DOM.W3C.HTML import HTMLSelectElement
from thug.DOM.W3C.HTML import HTMLSpanElement
from thug.DOM.W3C.HTML import HTMLStyleElement
from thug.DOM.W3C.HTML import HTMLTableCaptionElement
from thug.DOM.W3C.HTML import HTMLTableCellElement
from thug.DOM.W3C.HTML import HTMLTableColElement
from thug.DOM.W3C.HTML import HTMLTableElement
from thug.DOM.W3C.HTML import HTMLTableRowElement
from thug.DOM.W3C.HTML import HTMLTableSectionElement
from thug.DOM.W3C.HTML import HTMLTextAreaElement
from thug.DOM.W3C.HTML import HTMLTitleElement
from thug.DOM.W3C.HTML import HTMLUListElement
from thug.DOM.W3C.HTML import TAnimateColor

import logging

import bs4 as BeautifulSoup
from .Node import Node

log = logging.getLogger("Thug")


class DOMImplementation(HTMLDocument.HTMLDocument):
    features = ( ('core'        , '1.0'),
                 ('core'        , '2.0'),
                 ('core'        , None ),
                 ('html'        , '1.0'),
                 ('html'        , '2.0'),
                 ('html'        , None ),
                 ('events'      , '2.0'),
                 ('events'      , None ),
                 ('uievents'    , '2.0'),
                 ('uievents'    , None ),
                 ('mouseevents' , '2.0'),
                 ('mouseevents' , None ),
                 ('htmlevents'  , '2.0'),
                 ('htmlevents'  , None ),
                 ('views'       , '2.0'),
                 ('views'       , None ),
                 ('stylesheets' , '2.0'),
                 ('stylesheets' , None ))

    @staticmethod
    def hasFeature(feature, version):
        if version == "":
            version = None
        return (feature.lower(), version) in DOMImplementation.features

    TAGS = {
        "html"          : HTMLHtmlElement.HTMLHtmlElement,
        "head"          : HTMLHeadElement.HTMLHeadElement,
        "link"          : HTMLLinkElement.HTMLLinkElement,
        "title"         : HTMLTitleElement.HTMLTitleElement,
        "meta"          : HTMLMetaElement.HTMLMetaElement,
        "base"          : HTMLBaseElement.HTMLBaseElement,
        "isindex"       : HTMLIsIndexElement.HTMLIsIndexElement,
        "style"         : HTMLStyleElement.HTMLStyleElement,
        "body"          : HTMLBodyElement.HTMLBodyElement,
        "form"          : HTMLFormElement.HTMLFormElement,
        "select"        : HTMLSelectElement.HTMLSelectElement,
        "optgroup"      : HTMLOptGroupElement.HTMLOptGroupElement,
        "option"        : HTMLOptionElement.HTMLOptionElement,
        "input"         : HTMLInputElement.HTMLInputElement,
        "textarea"      : HTMLTextAreaElement.HTMLTextAreaElement,
        "button"        : HTMLButtonElement.HTMLButtonElement,
        "label"         : HTMLLabelElement.HTMLLabelElement,
        "fieldset"      : HTMLFieldSetElement.HTMLFieldSetElement,
        "legend"        : HTMLLegendElement.HTMLLegendElement,
        "ul"            : HTMLUListElement.HTMLUListElement,
        "ol"            : HTMLOListElement.HTMLOListElement,
        "dl"            : HTMLDListElement.HTMLDListElement,
        "dir"           : HTMLDirectoryElement.HTMLDirectoryElement,
        "menu"          : HTMLMenuElement.HTMLMenuElement,
        "li"            : HTMLLIElement.HTMLLIElement,
        "div"           : HTMLDivElement.HTMLDivElement,
        "p"             : HTMLParagraphElement.HTMLParagraphElement,
        "h1"            : HTMLHeadingElement.HTMLHeadingElement,
        "h2"            : HTMLHeadingElement.HTMLHeadingElement,
        "h3"            : HTMLHeadingElement.HTMLHeadingElement,
        "h4"            : HTMLHeadingElement.HTMLHeadingElement,
        "h5"            : HTMLHeadingElement.HTMLHeadingElement,
        "h6"            : HTMLHeadingElement.HTMLHeadingElement,
        "q"             : HTMLQuoteElement.HTMLQuoteElement,
        "blockquote"    : HTMLQuoteElement.HTMLQuoteElement,
        "span"          : HTMLSpanElement.HTMLSpanElement,
        "pre"           : HTMLPreElement.HTMLPreElement,
        "br"            : HTMLBRElement.HTMLBRElement,
        "basefont"      : HTMLBaseFontElement.HTMLBaseFontElement,
        "font"          : HTMLFontElement.HTMLFontElement,
        "hr"            : HTMLHRElement.HTMLHRElement,
        "ins"           : HTMLModElement.HTMLModElement,
        "del"           : HTMLModElement.HTMLModElement,
        "a"             : HTMLAnchorElement.HTMLAnchorElement,
        "object"        : HTMLObjectElement.HTMLObjectElement,
        "param"         : HTMLParamElement.HTMLParamElement,
        "img"           : HTMLImageElement.HTMLImageElement,
        "applet"        : HTMLAppletElement.HTMLAppletElement,
        "script"        : HTMLScriptElement.HTMLScriptElement,
        "frameset"      : HTMLFrameSetElement.HTMLFrameSetElement,
        "frame"         : HTMLFrameElement.HTMLFrameElement,
        "iframe"        : HTMLIFrameElement.HTMLIFrameElement,
        "table"         : HTMLTableElement.HTMLTableElement,
        "caption"       : HTMLTableCaptionElement.HTMLTableCaptionElement,
        "col"           : HTMLTableColElement.HTMLTableColElement,
        "colgroup"      : HTMLTableColElement.HTMLTableColElement,
        "thead"         : HTMLTableSectionElement.HTMLTableSectionElement,
        "tbody"         : HTMLTableSectionElement.HTMLTableSectionElement,
        "tfoot"         : HTMLTableSectionElement.HTMLTableSectionElement,
        "tr"            : HTMLTableRowElement.HTMLTableRowElement,
        "th"            : HTMLTableCellElement.HTMLTableCellElement,
        "td"            : HTMLTableCellElement.HTMLTableCellElement,
    }

    @staticmethod
    def createHTMLElement(doc, tag):
        if isinstance(tag, BeautifulSoup.NavigableString):
            return Node.wrap(doc, tag)

        if log.ThugOpts.Personality.isIE():
            if tag.name.lower() in ('t:animatecolor', ):
                return TAnimateColor.TAnimateColor(doc, tag)

        if tag.name.lower() in DOMImplementation.TAGS:
            return DOMImplementation.TAGS[tag.name.lower()](doc, tag)
        else:
            return HTMLElement.HTMLElement(doc, tag)



#!/usr/bin/env python

import PyV8

from .HTMLElement import HTMLElement
from .attr_property import attr_property
from .compatibility import thug_long


class HTMLImageElement(HTMLElement):
    def __init__(self, doc, tag):
        HTMLElement.__init__(self, doc, tag)

    align           = attr_property("align")
    alt             = attr_property("alt")
    border          = attr_property("border")
    height          = attr_property("height", thug_long)
    hspace          = attr_property("hspace", thug_long)
    isMap           = attr_property("ismap", bool)
    longDesc        = attr_property("longdesc")
    # Removed in DOM Level 2
    # lowSrc        = attr_property("lowsrc")
    name            = attr_property("name")
    #src            = attr_property("src")
    useMap          = attr_property("usemap")
    vspace          = attr_property("vspace", thug_long)
    width           = attr_property("width", thug_long)

    @property
    def complete(self):
        return True

    def getSrc(self):
        if 'src' in self.tag:
            return str(self.tag['src'])

        return None

    def setSrc(self, value):
        self.tag['src'] = str(value)

        if value.lower().startswith('res://'):
            onerror = getattr(self, 'onerror', None)

            if isinstance(onerror, PyV8.JSFunction):
                with self.doc.window.context as ctx:
                    onerror.__call__()

    src = property(getSrc, setSrc)



#!/usr/bin/env python
#
# Screen.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import logging
log = logging.getLogger("Thug")

from .JSClass import JSClass


class Screen(JSClass):
    def __init__(self, width = 800, height = 600, depth = 32):
        self._width  = width
        self._height = height
        self._depth  = depth
        self._left   = 0
        self._top    = 0

        self.__init_personality()

    def __init_personality(self):
        if log.ThugOpts.Personality.isIE():
            self.__init_personality_IE()
            return

        if log.ThugOpts.Personality.isFirefox():
            self.__init_personality_Firefox()
            return

        if log.ThugOpts.Personality.isChrome():
            self.__init_personality_Chrome()
            return

        if log.ThugOpts.Personality.isSafari():
            self.__init_personality_Safari()
            return

        if log.ThugOpts.Personality.isOpera():
            self.__init_personality_Opera()

    def __init_personality_IE(self):
        self.bufferDepth          = property(self._get_bufferDepth, self._set_bufferDepth)
        self.deviceXDPI           = self._deviceXDPI
        self.deviceYDPI           = self._deviceYDPI
        self.logicalXDPI          = self._logicalXDPI
        self.logicalYDPI          = self._logicalYDPI
        self.fontSmoothingEnabled = self._fontSmoothingEnabled
        self.updateInterval       = self._updateInterval

        if log.ThugOpts.Personality.browserMajorVersion >= 8:
            self.systemXDPI = self._systemXDPI
            self.systemYDPI = self._systemYDPI

        if log.ThugOpts.Personality.browserMajorVersion >= 9:
            self.pixelDepth = self._pixelDepth

    def __init_personality_Firefox(self):
        self.availLeft  = self._availLeft
        self.availTop   = self._availTop
        self.left       = self._left
        self.top        = self._top
        self.pixelDepth = self._pixelDepth

    def __init_personality_Chrome(self):
        self.availLeft  = self._availLeft
        self.availTop   = self._availTop
        self.pixelDepth = self._pixelDepth

    def __init_personality_Safari(self):
        self.availLeft  = self._availLeft
        self.availTop   = self._availTop
        self.pixelDepth = self._pixelDepth

    def __init_personality_Opera(self):
        self.pixelDepth = self._pixelDepth

    @property
    def availHeight(self):
        """
            The height of the screen (excluding the Windows Taskbar)
        """
        return self._height

    @property
    def availWidth(self):
        """
            The width of the screen (excluding the Windows Taskbar)
        """
        return self._width

    @property
    def colorDepth(self):
        """
            The bit depth of the color palette for displaying images/
            The color resolution (in bits per pixel) of the screen

        """
        return self._depth

    @property
    def height(self):
        """
            The total height of the screen
        """
        return self._height

    @property
    def _pixelDepth(self):
        """
            The color resolution (in bits per pixel) of the screen
        """
        return self._depth

    def _get_bufferDepth(self):
        return self._depth

    def _set_bufferDepth(self, value):
        try:
            self._depth = int(value)
        except ValueError:
            pass

    @property
    def width(self):
        """
            The total width of the screen
        """
        return self._width

    @property
    def _availLeft(self):
        """
            The first available pixel available from the left side
            of the screen
        """
        return self._left + 1

    @property
    def _availTop(self):
        """
            The first available pixel from the top of the screen
            available to the browser
        """
        return self._top + 1

    @property
    def _deviceXDPI(self):
        """
            Returns the current number of dots per inch (DPI) of
            the document's viewport along the horizontal (x) axis.
        """
        return 120

    @property
    def _deviceYDPI(self):
        """
            Returns the current number of dots per inch (DPI) of
            the document's viewport along the vertical (y) axis.
        """
        return 120

    @property
    def _logicalXDPI(self):
        """
            Returns the number of dots per inch (DPI) of the document's
            viewport along the horizontal (x) axis at normal zoom level.
        """
        return 96

    @property
    def _logicalYDPI(self):
        """
            Returns the number of dots per inch (DPI) of the document's
            viewport along the vertical (y) axis at normal zoom level.
        """
        return 96

    @property
    def _systemXDPI(self):
        """
            Returns the number of dots per inch (DPI) of the display
            screen along the horizontal (x) axis at normal zoom level.
        """
        return 120

    @property
    def _systemYDPI(self):
        """
            Returns the number of dots per inch (DPI) of the display
            screen along the vertical (y) axis at normal zoom level.
        """
        return 120

    @property
    def _fontSmoothingEnabled(self):
        """
            Returns a Boolean value that indicates whether font
            smoothing is enabled.
        """
        return False

    @property
    def _updateInterval(self):
        """
            Specifies or returns the time interval (in milliseconds)
            between screen updates.
        """
        return 0



#!/usr/bin/env python
#
# ThugOpts.py
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA  02111-1307  USA

import sys
import os
import logging

try:
    import urllib.parse as urlparse
except ImportError:
    import urlparse

from thug.DOM.Personality import Personality

log = logging.getLogger("Thug")


class ThugOpts(dict):
    proxy_schemes = ('http', 'socks4', 'socks5', )

    def __init__(self):
        self._verbose           = False
        self._debug             = False
        self._proxy             = None
        self._raise_for_proxy   = True
        self.local              = False
        self.extensive          = False
        self._threshold         = 0
        self._connect_timeout   = 10
        self._timeout           = 600
        self.ast_debug          = False
        self.http_debug         = 0
        self._useragent         = 'winxpie60'
        self._referer           = 'about:blank'
        self._events            = list()
        self._delay             = 0
        self._attachment        = False
        self._file_logging      = False
        self._json_logging      = False
        self._maec11_logging    = False
        self._es_logging        = False
        self._code_logging      = True
        self._cert_logging      = True
        self._no_fetch          = False
        self._broken_url        = False
        self._vt_query          = False
        self._vt_submit         = False
        self._vt_runtime_apikey = None
        self._mongodb_address   = None
        self._web_tracking      = False
        self._honeyagent        = True
        self.Personality        = Personality()

    def set_verbose(self, verbose):
        self._verbose = verbose

    def get_verbose(self):
        return self._verbose

    verbose = property(get_verbose, set_verbose)

    def set_debug(self, debug):
        self._debug = debug

    def get_debug(self):
        return self._debug

    debug = property(get_debug, set_debug)

    def set_proxy(self, proxy):
        p = urlparse.urlparse(proxy)

        if p.scheme.lower() not in self.proxy_schemes:
            log.warning('[ERROR] Invalid proxy scheme (valid schemes: http, socks4, socks5)')
            sys.exit(0)

        self._proxy = proxy

    def get_proxy(self):
        return self._proxy

    proxy = property(get_proxy, set_proxy)

    def set_raise_for_proxy(self, raise_for_proxy):
        self._raise_for_proxy = raise_for_proxy

    def get_raise_for_proxy(self):
        return self._raise_for_proxy

    raise_for_proxy = property(get_raise_for_proxy, set_raise_for_proxy)

    def get_useragent(self):
        return self._useragent

    def set_useragent(self, useragent):
        if useragent not in self.Personality:
            log.warning('[WARNING] Invalid User Agent provided (using default "%s")', self._useragent)
            return

        self._useragent = useragent

    useragent = property(get_useragent, set_useragent)

    def get_referer(self):
        return self._referer

    def set_referer(self, referer):
        self._referer = referer

    referer = property(get_referer, set_referer)

    def get_events(self):
        return self._events

    def set_events(self, events):
        for e in events.split(","):
            self._events.append(e.lower().strip())

    events = property(get_events, set_events)

    def get_delay(self):
        return self._delay

    def set_delay(self, timeout):
        try:
            _timeout = int(timeout)
        except ValueError:
            log.warning('[WARNING] Ignoring invalid delay value (should be an integer)')
            return

        self._delay = abs(_timeout)

    delay = property(get_delay, set_delay)

    def get_attachment(self):
        return self._attachment

    def set_attachment(self, attachment):
        self._attachment = attachment

    attachment = property(get_attachment, set_attachment)

    def get_file_logging(self):
        return self._file_logging

    def set_file_logging(self, file_logging):
        self._file_logging = file_logging

    file_logging = property(get_file_logging, set_file_logging)

    def get_json_logging(self):
        return self._json_logging

    def set_json_logging(self, json_logging):
        self._json_logging = json_logging

    json_logging = property(get_json_logging, set_json_logging)

    def get_maec11_logging(self):
        return self._maec11_logging

    def set_maec11_logging(self, maec11_logging):
        self._maec11_logging = maec11_logging

    maec11_logging = property(get_maec11_logging, set_maec11_logging)

    def get_es_logging(self):
        return self._es_logging

    def set_es_logging(self, es_logging):
        self._es_logging = es_logging

    elasticsearch_logging = property(get_es_logging, set_es_logging)

    def get_code_logging(self):
        return self._code_logging

    def set_code_logging(self, code_logging):
        self._code_logging = code_logging

    code_logging = property(get_code_logging, set_code_logging)

    def get_cert_logging(self):
        return self._cert_logging

    def set_cert_logging(self, cert_logging):
        self._cert_logging = cert_logging

    cert_logging = property(get_cert_logging, set_cert_logging)

    def get_no_fetch(self):
        return self._no_fetch

    def set_no_fetch(self, fetch):
        self._no_fetch = fetch

    no_fetch = property(get_no_fetch, set_no_fetch)

    def get_threshold(self):
        return self._threshold

    def set_threshold(self, threshold):
        try:
            value = int(threshold)
        except ValueError:
            log.warning('[WARNING] Ignoring invalid threshold value (should be an integer)')
            return

        self._threshold = value

    threshold = property(get_threshold, set_threshold)

    def get_connect_timeout(self):
        return self._connect_timeout

    def set_connect_timeout(self, timeout):
        try:
            seconds = int(timeout)
        except ValueError:
            log.warning('[WARNING] Ignoring invalid connect timeout value (should be an integer)')
            return

        self._connect_timeout = seconds

    connect_timeout = property(get_connect_timeout, set_connect_timeout)

    def get_timeout(self):
        return self._timeout

    def set_timeout(self, timeout):
        try:
            seconds = int(timeout)
        except ValueError:
            log.warning('[WARNING] Ignoring invalid timeout value (should be an integer)')
            return

        self._timeout = seconds

    timeout = property(get_timeout, set_timeout)

    def get_broken_url(self):
        return self._broken_url

    def set_broken_url(self, mode):
        self._broken_url = mode

    broken_url = property(get_broken_url, set_broken_url)

    def get_vt_query(self):
        return self._vt_query

    def set_vt_query(self):
        self._vt_query = True

    vt_query = property(get_vt_query, set_vt_query)

    def get_vt_submit(self):
        return self._vt_submit

    def set_vt_submit(self):
        self._vt_submit = True

    vt_submit = property(get_vt_submit, set_vt_submit)

    def get_vt_runtime_apikey(self):
        return self._vt_runtime_apikey

    def set_vt_runtime_apikey(self, vt_apikey):
        self._vt_runtime_apikey = vt_apikey

    vt_runtime_apikey = property(get_vt_runtime_apikey, set_vt_runtime_apikey)

    def get_web_tracking(self):
        return self._web_tracking

    def set_web_tracking(self, enabled):
        self._web_tracking = enabled

    web_tracking = property(get_web_tracking, set_web_tracking)

    def get_honeyagent(self):
        return self._honeyagent

    def set_honeyagent(self, enabled):
        self._honeyagent = enabled

    honeyagent = property(get_honeyagent, set_honeyagent)

    def get_mongodb_address(self):
        return self._mongodb_address

    def set_mongodb_address(self, mongodb_address):
        self._mongodb_address = mongodb_address

    mongodb_address = property(get_mongodb_address, set_mongodb_address)
